package tools;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.Arrays;

import util.FormatUtilities;

/*
 * Created on Oct 30, 2006
 */

/**
 * @author augsten
 */
public class CSVStatistic {

	public static final int T_NULL = -1;
	public static final int T_STRING = 0;
	public static final int T_DOUBLE = 1;
	public static final int T_INTEGER = 2;
	
	String[] fields;
	int[] fieldlen;		
	int[] strings;		
	int[] integers;
	long[] imin;
	long[] imax;
	int[] doubles;
	double[] dmin;
	double[] dmax;
	int[] emptystrings;
	int tuples;
	int[] nulls;
		
	/**
	 * 
	 */
	public CSVStatistic(String fileName, char sep, char quote) throws IOException {
		LineNumberReader in = 
			new LineNumberReader(new FileReader(fileName));
		
		String inline;
		// read field names
		if ((inline = in.readLine()) != null) {
			fields = FormatUtilities.getFields(inline, sep, quote);
			fieldlen = new int[fields.length];
			strings = new int[fields.length];
			integers= new int[fields.length];
			doubles = new int[fields.length];
			nulls = new int[fields.length];
			emptystrings = new int[fields.length];
			dmin = new double[fields.length];
			dmax = new double[fields.length];
			imin = new long[fields.length];
			imax = new long[fields.length];
			Arrays.fill(dmin, Double.MAX_VALUE);
			Arrays.fill(dmax, Double.MIN_VALUE);
			Arrays.fill(imin, Long.MAX_VALUE);
			Arrays.fill(imax, Long.MIN_VALUE);
			
			tuples = 0;
			while ((inline = in.readLine()) != null) {
				tuples++;
				String[] parse = FormatUtilities.getFields(inline, sep, quote);
				if (parse.length == 0) continue; // skip empty lines				
				for (int i = 0; i < Math.min(fieldlen.length, parse.length); i++) { // fields may be missing. surplus fields are ignored!
					fieldlen[i] = Math.max(fieldlen[i], parse[i].length());
					switch (getType(parse[i])) {
					case T_INTEGER:
						integers[i]++;
						long l = Long.parseLong(parse[i]);
						if (l < imin[i]) {
							imin[i] = l;
						}
						if (l > imax[i]) {
							imax[i] = l;
						}
						break;
					case T_DOUBLE:						
						doubles[i]++;
						double d = Double.parseDouble(parse[i]);
						if (d < dmin[i]) {
							dmin[i] = d;
						}
						if (d > dmax[i]) {
							dmax[i] = d;
						}
						break;
					case T_STRING:
					default:	
						strings[i]++;	
						if (parse[i].equals("")) {
							emptystrings[i]++;
						}
					}
				}
				for (int i = parse.length; i < fieldlen.length; i++) {
					nulls[i]++;
				}
			}
		}
	}

	public static int getType(String s) {
		if (s == null) {
			return T_NULL;
		}
		if (s.equals("")) {
			return T_STRING;
		}
		try {
			long l = Long.parseLong(s);
			System.out.println(l); // DELETE
			return T_INTEGER;
		} catch (NumberFormatException e) {
			char c = s.toLowerCase().charAt(s.length() - 1);
			// do not support trailing format specifiers, e.g. 6d is a string, not the double 6.0
			if ((c == 'd') || (c == 'f')) { 
				return T_STRING;
			}
			try {
				double d = Double.parseDouble(s);
				System.out.println(d); // DELETE
				return T_DOUBLE;
			} catch (NumberFormatException f) {
				return T_STRING;
			}
		}
	}
	
	
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer s = new StringBuffer();
		for (int i = 0; i < fields.length; i++) {
			s.append(fields[i] + ":\n");
			if (integers[i] != 0) {
				s.append("  " + integers[i] + " integers (" + (100.0*integers[i]/tuples) + "%) min=" + imin[i] + ", max=" + imax[i] + "\n");
			}
			if (doubles[i] != 0) {
				s.append("  " + doubles[i] + " doubles (" + (100.0*doubles[i]/tuples) + "%) min=" + dmin[i] + ", max=" + dmax[i] + "\n");
			}
			if (strings[i] != 0) {
				s.append("  " + strings[i] + " strings (" + (100.0*strings[i]/tuples) + "%), max string length=" + fieldlen[i] + "\n");
				s.append("  --> " + emptystrings[i] + " of them are empty (" + (100.0*emptystrings[i]/strings[i]) + "%)\n");
			}
			if (nulls[i] != 0) {
				s.append("  " + nulls[i] + " NULL values (" + (100.0*nulls[i]/tuples) + "%)\n");
			}
		}
		s.append("\nnumber of tuples: " + tuples + "\n");
		return s.toString();
	}
	public static void main(String[] args) throws IOException {
		// DELETE TO COMMENT!	
		int t = getType(args[0]);
		switch (t) {
		case T_NULL: System.out.println("NULL"); break;
		case T_STRING: System.out.println("STRING"); break;
		case T_INTEGER: System.out.println("INTEGER"); break;
		case T_DOUBLE: System.out.println("DOUBLE"); break;
		default: System.out.println("UNKNOWN");
		}		
		System.out.println(">>" + args[0] + "<<");
		// UNCOMMENT
//		if (args.length != 1) {
//			System.out.println("Give me a CSV (comman separated values) file,\nand I will give you statistics about the values in the columns");
//			System.out.println("\n  params: filename\n");
//			System.out.println("I assume that fields are separated with ';'.\nDouble quotes (\"...\") and spaces around strings are dropped.");
//			System.out.println();
//			System.exit(-1);
//		}
//		System.out.println(new CSVStatistic(args[0], ';', '"'));
	}
}
