package tools;

import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.sql.Connection;
import java.sql.SQLException;

import mysqltools.MySqlInsertBuffer;
import mysqltools.MySqlTools;

import sqltools.SQLTools;
import sqltools.SimpleTableWrapper;
import sqltools.TableWrapper;
import util.FormatUtilities;
import configuration.*;
/*
 * Created on Oct 26, 2006
 */

/**
 * @author augsten
 */
public class CSVtoTbl {

	private CSVtoTbl() {}

	static CSVStatistic statistic;
	static String tblName, fileName;
	static char sep, quote;

	public static void evaluateArgs(String[] args) {
		if ((args.length >= 2) && (args.length <= 4)) {
			fileName = args[0];
			tblName = args[1];
			sep = ';';
			quote = '"';
			if (args.length == 3) {
				sep = args[2].charAt(0);
				if (args[2].equals("tab")) sep = '\t'; 
			} 
			if (args.length == 4) {
				quote = args[3].charAt(0);
			}
		} else {
			System.err.println("Reads CVS file into table.");
			System.err.println("\n  params: fileName tblName [sep [quote]]\n");
			System.err.println("  -> sep: column-separator, default is ';' ('tab' is tabulator)");
			System.err.println("  -> quote: quote character for string columns, default is '\"'");
			System.err.println();
			System.exit(-1);
		}
		System.out.println("Importing from '" + fileName + "'\n   into table '" 
				+ tblName + "'\n   assuming fields beeing separated by " + ((sep == '\t') ? "tabs" : "'" + sep + "'")  + ".");
	}
	
	public static TableWrapper getTable(String fileName, String tblName, Connection con) throws IOException {
		LineNumberReader in = 
			new LineNumberReader(new FileReader(fileName));
				
//		String inline;
//		String[] fields;
//		// read field names
//		if ((inline = in.readLine()) == null) {
//			return null;
//		}
//		fields = FormatUtilities.getFields(inline, sep, quote);
//		int[] charlen = new int[fields.length]; 
//		
//		while ((inline = in.readLine()) != null) {
//			String[] parse = FormatUtilities.getFields(inline, sep, quote);
//			for (int i = 0; i < Math.min(charlen.length, parse.length); i++) { // fields may be missing. surplus fields are ignored!
//				charlen[i] = Math.max(charlen[i], parse[i].length());
//			}
//		}
		
		statistic = new CSVStatistic(fileName, sep, quote);
		int fieldlen[] = statistic.fieldlen;
		String[] fields = statistic.fields;

		StringBuffer atbList = new StringBuffer();
		StringBuffer qry = new StringBuffer("CREATE TABLE `" + tblName + "` (\n");
		for (int i = 0; i < fieldlen.length; i++) {
			atbList.append("`" + fields[i] + "`");
			qry.append("  `" + fields[i] + "`" + " CHAR(" + fieldlen[i] + ")");
			if (i != fieldlen.length - 1) {
				atbList.append(",");
				qry.append(",\n");				
			} else {
				qry.append(")");
			}
		}				
		in.close();
		
		return 
			new SimpleTableWrapper(con, con, 
					new MySqlInsertBuffer(), 
					tblName, atbList.toString(), qry.toString());		
	}
	
	public static int scanFile(String fileName, TableWrapper tbl) throws IOException, SQLException {
		LineNumberReader in = 
			new LineNumberReader(new FileReader(fileName));
		
		// count fields
		int fields = FormatUtilities.getFields(in.readLine(), sep, quote).length;
		
		// scan rest
		String inline;
		int count = 0;
		while ((inline = in.readLine()) != null) {
			String[] parse = FormatUtilities.getFields(inline, sep, quote);
			if (parse.length == 0) continue; // skip empty lines				
			StringBuffer tuple = new StringBuffer("(");			
			for (int i = 0; i < fields; i++) { // fields may be missing. surplus fields are ignored!				
				if (i < parse.length) {
					tuple.append("'" + SQLTools.escapeSingleQuote(FormatUtilities.stripQuotes(parse[i], '"')) + "'");
				} else {
					tuple.append("NULL");
				}
				if (i < fields - 1) {
					tuple.append(",");
				} else {
					tuple.append(")");
				}
			}
			tbl.getInsBuff().insert(tuple.toString());
			count++;
		}
		tbl.getInsBuff().close();
		return count;
	}
	
	public static void main(String[] args) throws Exception {
		Configuration config = new Configuration("." + CSVtoTbl.class.getSimpleName());
		config.loadDrivers();
		Connection con = config.getConnection();
		System.out.println();
		evaluateArgs(args);
		TableWrapper tbl = getTable(fileName, tblName, con);
		tbl.reset();
		int tuples = scanFile(fileName, tbl);
		MySqlTools.addTblComment(tbl, "imported from CSV-file '" + fileName + "'");
		System.out.println("\n" + statistic);
	}
	
}


