package tools;

import intervalenc.IntervalEncForest;

import java.sql.Connection;
import java.sql.SQLException;

import mysqltools.MySqlTools;
import sqltools.SQLTools;
import sqltools.TableWrapper;
import tree.ForestConversion;
import tupleperpath.TuplePerPath;
import configuration.*;
public class ExtractAddressTrees {
	
	static String tblAddress, tblTree, atbTreeID, usage;
	static String[] atbLvl;
	static boolean force = false;

	public static void evaluateArgs(String[] args) {
		usage =
			"\nparams: [-f] tblAddress tblTree atbTreeID atbLvl0 ... atbLvlN\n\n"
			+ "   -f ............ reset existing table tblStreets without asking\n"
			+ "   tblAddress .... table containing all addresses\n"
			+ "   tblTree ....... table that will contain address trees (will be created)\n"
			+ "   atbTreeID ..... street ID (attribute in tblAddresses)\n"
			+ "   atbLvlX ....... tree level X of address tree, level(root)=0 (attribute in tblAddresses)\n";
		if (args.length < 4) {
			System.err.print(usage);
			System.exit(-1);
		}
		int j = 0;
		for (int i = 0; i < args.length; i++) {
			args[j] = args[i];
			if (args[i].equals("-f")) {
				force = true;
			} else {
				j++;
			}
		}
		tblAddress = args[0];
		tblTree = args[1];		
		atbTreeID = args[2];
		int levels = j - 3;
		atbLvl = new String[levels];
		for (int i = 0; i < levels; i++) {
			atbLvl[i] = args[i + 3];
		}
	}
	
	public static void addTblComment(TableWrapper tw, String className, String[] args) throws SQLException {
		StringBuffer comment = new StringBuffer("cmd: " + className ); 
		for (int i = 0; i < args.length; i++) {
			if (!args[i].equals("-f")) comment.append(" " + args[i]);
		}
		MySqlTools.addTblComment(tw, comment.toString());
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {

		SQLTools.DEBUG = false;
		evaluateArgs(args);		
		Configuration config = new Configuration("." + ExtractAddressTrees.class.getSimpleName());
		config.loadDrivers();
		Connection con = config.getConnection();

		TuplePerPath tpp = new TuplePerPath(con, tblAddress, atbTreeID, atbLvl);
		tpp.setSortType(new String[] {null, "UNSIGNED", null, "UNSIGNED"});
			
		if (!tpp.hasTreeIDs()) {
			System.out.println("WARNING:");
			System.out.println("Adding column '" + atbTreeID 
					+ "' with street IDs to input table '" + tpp.getTblName() + "'");
			tpp.insertTreeIDs();
			System.out.println(tpp.getTreeIDs().length + " streets found.");
		}
		
		IntervalEncForest ie = new IntervalEncForest(con, tblTree);
		if ((!force) && (ie.exists())) {
			System.out.println("Table '" + ie.getTblName() 
					+ "' exists. Use '-f' to reset.\n" + usage);
			System.exit(-2);
		}
		ie.reset();
		ForestConversion.convertCleanNulls(tpp, ie);
		
		String qry =
			"UPDATE `" + ie.getTblName() + "` SET " + ie.getAtbLabel() + "='XXX' WHERE "
			+ ie.getAtbLft() + "=0";
		SQLTools.execute(ie.getStatement(), qry, "Overwriting roots of address trees with 'XXX' (table '" + ie.getTblName() + "'");
		ie.createPreorderIndex();
		
		addTblComment(ie, "ExtractAddressTrees", args);
	}

}
