/**
 * 
 */
package tools;

import java.sql.ResultSet;
import java.sql.SQLException;

import mysqltools.MySqlTools;
import sqltools.SQLTools;
import sqltools.SimpleTableWrapper;
import sqltools.TableWrapper;
import string.StringTbl;
import configuration.*;
/**
 * @author naugsten
 *
 */
public class ExtractStreetNames {

	static String tblNameIn, tblNameOut;
	static String atbStrID, atbStrName, atbStrType;
	static boolean force = false;

	
	public static void evaluateArgs(String[] args) {
		String usage =
			"\nparams: [-f] tblAddresses tblStreets atbID atbName [atbType]\n\n"
			+ "   -f............ reset existing table tblStreets without asking\n"
			+ "   tblAddresses.. table containing all addresses\n"
			+ "   tblStreets.... table that will contain street names (will be created)\n"
			+ "   atbID......... street ID (attribute name in tblAddresses)\n"
			+ "   atbName....... street name (attribute name in tblAddresses)\n"
			+ "   atbType....... street type (attribute name in tblAddresses)\n\n";
		if (args.length < 4) {
			System.err.print(usage);
			System.exit(-1);
		}
		int j = 0;
		for (int i = 0; i < args.length; i++) {
			args[j] = args[i];
			if (args[i].equals("-f")) {
				force = true;
			} else {
				j++;
			}
		}
		tblNameIn = args[0];
		tblNameOut = args[1];
		atbStrID = args[2];
		atbStrName = args[3];
		if (j == 5) {
			atbStrType = args[4];
		}
	}
	
	public static void addTblComment(TableWrapper tw, String className, String[] args) throws SQLException {
		StringBuffer comment = new StringBuffer("cmd: " + className ); 
		for (int i = 0; i < args.length; i++) {
			if (!args[i].equals("-f")) comment.append(" " + args[i]);
		}
		MySqlTools.addTblComment(tw, comment.toString());
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {

		SQLTools.DEBUG = false;
		evaluateArgs(args);		
		Configuration config = new Configuration("." + ExtractStreetNames.class.getSimpleName());
		config.loadDrivers();

		StringTbl strTbl = new StringTbl(config.getConnection(), tblNameOut);
		if ((!force) && (strTbl.exists())) {
			System.err.println("Table '" + strTbl.getTblName() 
					+ "' exists. Use '-f' to overwrite existing table.");
			System.exit(-2);
		}
		strTbl.reset();
		
		String qry =
			"SELECT DISTINCT `" + atbStrID + "`,`" + atbStrName + "`" 
			+ ((atbStrType == null) ? "" : ",`" + atbStrType + "`") 
			+ " FROM `" + tblNameIn + "`";
		ResultSet rs = SQLTools.executeQuery(strTbl.getStatement(), qry, 
				"Loading street names from table '" + tblNameIn + "'.");
		int rows = 0;
		while (rs.next()) {			
			long strID = rs.getLong(atbStrID);
			String strName = rs.getString(atbStrName);
			if (rs.getMetaData().getColumnCount() == 3) {
				String strType = rs.getString(atbStrType);
				if (strType.trim().length() > 0) {
					strName = rs.getString(atbStrType) + " " + strName;
				}
			}
			strTbl.getInsBuff().insert("(" + strID + ",'" 
					+ SQLTools.escapeSingleQuote(strName) + "')");
			rows++;
		}
		System.out.println("Written " + rows + " streets.");
		strTbl.flush();
		addTblComment(strTbl, "ExtractStreetNames", args);
	}

}
