package tools;
import intervalenc.IntervalEncForest;

import java.io.IOException;
import java.util.Iterator;

import tree.LblValTree;
import tree.MMForest;
import tree.LblTree;
import configuration.*;
/*
 * Created on Feb 1, 2005
 */

/**
 * Given a forest and a depth, all treeIDs of trees with given depth are
 * printed to std-out.
 * 
 * @author augsten
 */
public class GetForestTblInfo {
	
	public static final String MSG_USAGE 
		= "params: tblName \n";

	public static void main(String[] args) throws Exception {
		int argcnt = 1;
		if (args.length != argcnt) {
			System.err.println(MSG_USAGE);
		} else {
			Configuration config = new Configuration("." + GetForestTblInfo.class.getSimpleName());
			config.loadDrivers();
			
			// initialize forests
			String tblName = args[0];
			IntervalEncForest tblForest = new IntervalEncForest(config.getConnection(), 
					config.getConnection(), 
					config.getInsertBuffer(), tblName);
				

			int nodes = 0;
			int depth = 0;
			int leaves = 0;
			int nodesMin = Integer.MAX_VALUE;
			int nodesMax = Integer.MIN_VALUE;
			int depthMin = Integer.MAX_VALUE;
			int depthMax = Integer.MIN_VALUE;
			int N = 0;

			for (Iterator<LblValTree> it = tblForest.forestIterator(); it.hasNext();) {
				LblValTree t = it.next();
				N++;
				int treeNodes = t.getNodeCount();
				nodes += treeNodes;
				depth += t.getDepth();
				leaves += t.getLeafCount();
				depthMin = (t.getDepth() < depthMin ? t.getDepth() : depthMin);
				depthMax = (t.getDepth() > depthMax ? t.getDepth() : depthMax);
				nodesMin = (treeNodes < nodesMin ? treeNodes : nodesMin);
				nodesMax = (treeNodes > nodesMax ? treeNodes : nodesMax);
			}
			System.out.println("Forest '" + tblName + "'");
			System.out.println(
					"number of trees: " + N + "\n" + 
					"total number of nodes: " + nodes + "\n" +
					"total number of leaves: " + leaves + "\n" +
					"min nodes per tree: " + nodesMin + "\n" +
					"av. nodes per tree: " + (double)nodes / N + "\n" +
					"max nodes per tree: " + nodesMax + "\n" +
					"av. leaves per tree: " + (double)leaves / N + "\n" +
					"min depth: " + depthMin + "\n" +
					"av. depth: " + (double)depth / N + "\n" +
					"max depth: " + depthMax 
					);
		}
	}
}
