package tools;
/*
 * Created on Feb 1, 2005
 */

/**
 * Give a forset file, the ID of the tree and you will get a description
 * plus a print of the tree in various formats on stdout.
 * 
 * @author augsten
 */

import tree.MMForest;
import tree.LblTree;
import distance.PQGramDist;
import distance.VectorProfile;

import java.io.IOException;
import java.util.Arrays;
import java.util.Enumeration;

public class GetTreeFileInfo {
	
	public static final String MSG_USAGE 
		= "params: [s|n|p|v] forestfile treeID\n" +
				"        s - statistics only\n" +
				"        n - normal display and statistics\n" +
				"        p - prettyprint and statistics\n" +
				"        g - show pqgrams\n" + 
				"        v - verbose: prettyprint, normalprint and statistics\n"; 

	public static void main(String[] args) throws IOException {
		int argcnt = 3;
		if (args.length != argcnt) {
			System.err.println(MSG_USAGE);
		} else {
			char option = args[0].charAt(0);
			String forestfile = args[1];
			int treeID = Integer.parseInt(args[2]);
			MMForest f = new MMForest(forestfile);			
			int[] treeIDs = new int[] {treeID};
			MMForest sf = f.getSubforest(treeIDs);
			if (sf.size() != 1) {
				System.err.println(sf.size() + " trees found with ID " + 
						treeID + ".");
			} else {
				LblTree t = sf.getTreeAt(0);
				int nodes = t.getNodeCount();
				int leaves = t.getLeafCount();
				double mfanout = t.getMediumFanout();
				int maxdepth = t.getDepth();				
				String root = t.getLabel();

				// choose output format
				switch (option) { 
				case 'g':
					PQGramDist pqDist = new PQGramDist(2, 3, false);
					VectorProfile g = (VectorProfile)pqDist.getProfile(t);
					System.out.println(g);
				case 'v':
					System.out.println("\n--- PRETTY PRINT ---");
					t.prettyPrint();
				case 'n':
					System.out.println("\n--- STRING ENCODING ---");
					System.out.println(t);
					break;
				case 'p':
					System.out.println("\n--- PRETTY PRINT ---");
					t.prettyPrint();
					break;
				case 's':
				default:	
				}
				
				System.out.println("\n--- STATISTICS ---");
				
				System.out.println("tree nr. " + treeID + ": '" + root + "'");
				System.out.println("nodes: " + nodes + " (" + leaves + ")");
				System.out.println("depth: " + maxdepth);
				System.out.println("medium fanout: " + mfanout);
				
				// get leaves and nodes per level
				int[] lnodes = new int[maxdepth + 1];
				int[] lleaves = new int[maxdepth + 1];
				Arrays.fill(lnodes, 0);
				Arrays.fill(lleaves, 0);
				for (Enumeration e = t.breadthFirstEnumeration(); e.hasMoreElements();) { 
					LblTree n = (LblTree)e.nextElement();
					int level = n.getLevel();
					lnodes[level]++;
					if (n.isLeaf()) {
						lleaves[level]++;
					}
				}
				System.out.println("\nleaves and nodes per level:");
				for (int i = 0; i < lnodes.length; i++) {
					System.out.println("level " + i + ": " + 
							lnodes[i] + " nodes (" + lleaves[i] + " leaves)");
				}
				
			}
		}
	}
}
