package tools;

import java.sql.Connection;
import java.sql.SQLException;

import mysqltools.MySqlTools;

import tree.Forest;
import tree.ForestConversion;
import tupleperpath.TuplePerPath;
import adjacencyenc.AdjacencyEncForest;
import intervalenc.IntervalEncForest;
import configuration.*;
public class TreeFormatConverter {

	static String fromType;
	static String toType;
	static String tblNameFrom;
	static String tblNameTo;
	static String atbTreeID;
	static String[] atbLevel;
	static String[] sortType;
	static Configuration config;
	
	public static Forest getTable(String type, String tblName, String atbTreeID, String[] atbLevel, boolean reset) 
	throws Exception {
		Forest dba;
		Connection con = config.getConnection();
		if (type.equals("ie")) {			
			dba = new IntervalEncForest(con, tblName);
			if (reset) ((IntervalEncForest)dba).reset();
		} else if (type.equals("ae")) {
			dba = new AdjacencyEncForest(con, tblName);
			if (reset) ((AdjacencyEncForest)dba).reset();
		} else if (type.equals("tpp")) {
			dba = new TuplePerPath(con, tblName, atbTreeID, atbLevel);
			((TuplePerPath)dba).setSortType(sortType);
			if (reset) ((TuplePerPath)dba).reset();
		} else {
			System.err.println("Unknown fromType '" + type + "'");
			dba = null;
		}
		return dba;
	}	
	
	public static void showParams() {
		System.err.println("Convert the storage format of a table containing trees.\n");
		System.err.println("params: fromType toType tblSrc tblDest [atbTreeID atbLvl1 [atbLvl2 [...]]]\n");
		System.err.println("   fromType/toType:");			
		System.err.println("     ie: interval encoding");			
		System.err.println("     ae: adjacency list encoding");			
		System.err.println("     tpp: tuple-per-path encoding\n");
		System.err.println("If one of the types is tpp, " + "\n * also atbTreeID and atbLvl1..atbLvlN must be given");
		System.err.println("If formType is tpp, then");
		System.err.println(" * unnecessary null values are cleaned;");		
		System.err.println(" * atbLvl2 and atbLvl4 are sorted as UNSIGNED");		
	}
	
	public static void getParams(String[] args) {
		int argsCnt = 4;
		if (args.length < argsCnt) {
			showParams();
			System.exit(-1);
		}
		if (args.length >= argsCnt) {
			fromType = args[0];
			toType = args[1];
			tblNameFrom = args[2];
			tblNameTo = args[3];
			if (fromType.equals("tpp") || toType.equals("tpp")) {
				if (args.length >= argsCnt + 2) {
					atbTreeID = args[4];
					atbLevel = new String[args.length - 5]; 
					sortType = new String[args.length - 5]; 
					for (int i = 5; i < args.length; i++) {
						atbLevel[i - 5] = args[i];
						sortType[i - 5] = null;
					}
					sortType[1] = "UNSIGNED";
					sortType[3] = "UNSIGNED";					
				} else {
					showParams();
					System.exit(-1);
				}
			}
		}

	}	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		config = new Configuration("." + TreeFormatConverter.class.getSimpleName());
		config.loadDrivers();
		getParams(args);	
		Forest dbaFrom = getTable(fromType, tblNameFrom, atbTreeID, atbLevel, false);
		Forest dbaTo = getTable(toType, tblNameTo, atbTreeID, atbLevel, true);
		if (fromType.equals("tpp")) {			
			ForestConversion.convertCleanNulls(dbaFrom, dbaTo);
		} else {
			ForestConversion.convert(dbaFrom, dbaTo);			
		}
	}
}
