/*
 * Created on Oct 6, 2005
 */
package vldb2006;

import java.sql.Connection;
import java.sql.ResultSet;

import mysqltools.MySqlInsertBuffer;
import sqltools.SQLTools;
import adjacencyenc.AdjacencyEncForest;
import adjacencyenc.AdjNode;
import java.sql.SQLException;
import configuration.*;
/**
 * @author augsten
 */
public class AdjTreeStatistics {
	
	public static void printStatistics(AdjacencyEncForest f, int treeID) 
		throws SQLException {

		AdjNode root = f.getRootNode(treeID);
		if (root == null) {
			System.out.println("Tree " + treeID + " not found in `" + f.getTblName() + "`");
			System.exit(-1);
		}
		AdjacencyEncForest f0 = new AdjacencyEncForest(f.getCon(), f.getStreamCon(),
				new MySqlInsertBuffer(), "f_stat_0");
		f0.reset();
		f0.createIndices();
		
		// write root node to f0
		f0.getInsBuff().insert("(" +
				root.getTreeID() + "," +
				root.getNodeID() + "," +
				root.getSibPos() + "," +
				root.getParentID() + "," +
				"'" + SQLTools.escapeSingleQuote(root.getLabel()) + "')");
		f0.getInsBuff().flush();
		
		// SQL
		ResultSet rs;
		String F = "`" + f.getTblName() + "`";
		
		// statistics
		int sumFanout = 0;
		int sumVariance = 0;
		int sumNonLeaf = 0;
		int fanoutMin = Integer.MAX_VALUE;
		int fanoutMax = 0;
		int level = 0;
		
		boolean levelFound = true;
		while (levelFound) {
			level++;
			AdjacencyEncForest f1 = new AdjacencyEncForest(f.getCon(), f.getStreamCon(),
					new MySqlInsertBuffer(), "f_stat_" + level);
			f1.reset();
			f1.createIndices();
			String F0 = "`" + f0.getTblName() + "`";
			String F1 = "`" + f1.getTblName() + "`";
			String qry = 
				"INSERT INTO " + F1 + " "
				+ "SELECT " + F + ".* FROM " +  F0 + "," + F + " " 
				+ "WHERE " 
				+ F + "." + f.getAtbTreeID() + "=" + F0 + "." + f0.getAtbTreeID() + " AND "				
				+ F + "." + f.getAtbParentID() + "=" + F0 + "." + f0.getAtbNodeID();				
			SQLTools.execute(f.getStatement(), qry, 
					"Computing nodes of level " + level);
			qry = "SELECT " + f1.getAtbParentID() + ",COUNT(*) AS cnt"
				+ " FROM " + f1.getTblName()
				+ " WHERE " + f1.getAtbTreeID() + "=" + treeID
				+ " GROUP BY " + f1.getAtbParentID();
			rs = SQLTools.executeQuery(f1.getStreamStatement(), qry,
					"Computing fanout of nodes at level " + (level - 1));
			levelFound = false;
			
			while (rs.next()) {
				levelFound = true;
				int fanout = rs.getInt("cnt");
				sumFanout += fanout;
				sumVariance += (fanout * fanout);
				if (fanoutMin > fanout) fanoutMin = fanout;
				if (fanoutMax < fanout) fanoutMax = fanout;
				sumNonLeaf++;
			}
			f0.drop();
			f0 = f1;			
		}
		f0.drop();
		
		String qry = "SELECT COUNT(*) AS cnt FROM " + F + " WHERE " +
			f.getAtbTreeID() + "=" + treeID;
		rs = SQLTools.executeQuery(f.getStatement(), qry, "Counting nodes");
		rs.next();	
		int sumNodes = rs.getInt("cnt");
		
		System.out.println("TREE NR. " + treeID + " OF TABLE " + F + ":");
		System.out.println("  depth: " + (level - 1));
		System.out.println("NODES");
		System.out.println("  total: " + sumNodes);
		System.out.println("  non-leaves: " + sumNonLeaf 
				+ " (=" + ((double)sumNonLeaf / sumNodes * 100) + "%)");
		System.out.println("  leaves: " + (sumNodes - sumNonLeaf) 
				+ " (=" + ((double)(sumNodes - sumNonLeaf) / sumNodes * 100) + "%)");
		System.out.println("FANOUT OF NON-LEAVES:");
		if (fanoutMax != 0) {
			System.out.println("  min: " + fanoutMin);
			System.out.println("  max: " + fanoutMax);
			double mean = ((double)sumFanout/sumNonLeaf);
			double rms = Math.sqrt(((double)sumVariance/sumNonLeaf) - 
					(mean * mean));
			System.out.println("  avg: " + mean
					+ " +/- " + rms);
		} else {
			System.out.println("  no non-leaf in tree");
		}
		
	}

	/**
	 * 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		
		if (args.length != 2) {
			System.err.println("  Print some statistics for the adjacency-list-encoded tree <treeID> in table <tblName>.");
			System.err.println("  params: tblName treeID");
			System.exit(-1);
		}
		String tblName = args[0];
		int treeID = Integer.parseInt(args[1]);
		
		// connect to database
		// connect to database
		Connection[] con = Configuration.getCons(2);
		
		AdjacencyEncForest f = new AdjacencyEncForest(con[0], con[1],
				new MySqlInsertBuffer(), tblName);
		
		SQLTools.DEBUG = false;
		
		AdjTreeStatistics.printStatistics(f, treeID);

	}
}