/*
 * Created on Sep 30, 2005
 */
package vldb2006;

import index.DeltaProfile;
import index.Index;

import java.sql.*;

import mysqltools.MySqlInsertBuffer;
import adjacencyenc.AdjacencyEncForest;
import sqltools.*;
import configuration.*;
/**
 * @author Varunesh
 */
public class BuildIndexForForest {
	
	public static void BuildIndex(String args[]) throws Exception
	{
		if (args.length != 3) {
			System.err.println("  Builds a pq-gram index for the forest in table <tblName>.");
			System.err.println("  params: tblName p q");
			System.exit(-1);
		}
		String tblName = args[0];
		int p = Integer.parseInt(args[1]);
		int q = Integer.parseInt(args[2]);
		
		// connect to database
		// connect to database
		Connection[] con = Configuration.getCons(2);
		
		AdjacencyEncForest f = new AdjacencyEncForest(con[0], con[1],
				new MySqlInsertBuffer(), tblName);
		
		SQLTools.DEBUG = false;
		
		String idxName = tblName + "_Idx1_" + p + q;
		Index idx = new Index(idxName, con[0], p, q, new hash.PrefixHash(2));
	//	Index idx = new Index(idxName, con[0], p, q, new hash.StringHash(4));
		
		System.out.println("# Building " + p + "," + q + "-gram index `"
				+ idxName + "` for `" + tblName + "`...");
		System.out.println("# treeID, treeSize [nodes], time [ms]");		
		
		idx.reset();		
		idx.addIndex();		
		String qry =
			"SELECT " + f.getAtbTreeID() + ", COUNT(" + f.getAtbNodeID() + ") AS cnt FROM `" 
			+ f.getTblName() + "` GROUP BY " + f.getAtbTreeID();
		ResultSet trees = SQLTools.executeQuery(f.getStatement(), qry,
				"Selecting treeIDs from " + f.getTblName() + ".");
		long overall_start = System.currentTimeMillis();
		while (trees.next()) {
			long start = System.currentTimeMillis();
			int treeID =  trees.getInt(f.getAtbTreeID());
			int size = trees.getInt("cnt");
			DeltaProfile deltaProf = 
				new DeltaProfile(f, p, q, idx.getHf(), 
						"P_tmp", "Q_tmp", new MySqlInsertBuffer(), true);
			deltaProf.createProfile(treeID);
			idx.add(deltaProf);
			deltaProf.drop();
			System.out.println(treeID + " " + size + " " + (System.currentTimeMillis() - start));
		}
		System.out.println("# Overall time elapsed: " + (System.currentTimeMillis() - overall_start) + "ms");
		
		
		
	}
	
	
	/**
	 * 
	 * @param args
	 * @throws Exception
	 */
	
	
	public static void main(String[] args) throws Exception {
		
		if (args.length != 3) {
			System.err.println("  Builds a pq-gram index for the forest in table <tblName>.");
			System.err.println("  params: tblName p q");
			System.exit(-1);
		}
		String tblName = args[0];
		int p = Integer.parseInt(args[1]);
		int q = Integer.parseInt(args[2]);
		
		// connect to database
		// connect to database
		Connection[] con = Configuration.getCons(2);
		
		AdjacencyEncForest f = new AdjacencyEncForest(con[0], con[1],
				new MySqlInsertBuffer(), tblName);
		
		SQLTools.DEBUG = false;
		
		String idxName = tblName + "_Idx_" + p + q;
		Index idx = new Index(idxName, con[0], p, q, new hash.PrefixHash(2));
	//	Index idx = new Index(idxName, con[0], p, q, new hash.StringHash(4));
		
		System.out.println("# Building " + p + "," + q + "-gram index `"
				+ idxName + "` for `" + tblName + "`...");
		System.out.println("# treeID, treeSize [nodes], time [ms]");		
		
		idx.reset();		
		idx.addIndex();		
		String qry =
			"SELECT " + f.getAtbTreeID() + ", COUNT(" + f.getAtbNodeID() + ") AS cnt FROM `" 
			+ f.getTblName() + "` GROUP BY " + f.getAtbTreeID();
		ResultSet trees = SQLTools.executeQuery(f.getStatement(), qry,
				"Selecting treeIDs from " + f.getTblName() + ".");
		long overall_start = System.currentTimeMillis();
		while (trees.next()) {
			long start = System.currentTimeMillis();
			int treeID =  trees.getInt(f.getAtbTreeID());
			int size = trees.getInt("cnt");
			DeltaProfile deltaProf = 
				new DeltaProfile(f, p, q, idx.getHf(), 
						"P_tmp", "Q_tmp", new MySqlInsertBuffer(), true);
			deltaProf.createProfile(treeID);
			idx.add(deltaProf);
			deltaProf.drop();
			System.out.println(treeID + " " + size + " " + (System.currentTimeMillis() - start));
		}
		System.out.println("# Overall time elapsed: " + (System.currentTimeMillis() - overall_start) + "ms");
	}
}
