/*
 * Created on Oct 5, 2005
 */
package vldb2006;
import configuration.*;
import java.io.*;

import index.DeltaProfile;
import index.Index;

import java.sql.Connection;
import java.sql.ResultSet;

import mysqltools.MySqlInsertBuffer;
import sqltools.SQLTools;
import tree.LblTree;
import tree.LblValTree;
import adjacencyenc.AdjacencyEncForest;

/**
 * @author Varunesh Mishra
 */
public class Lookup {

	/**
	 * 
	 * @param args
	 * @throws Exception
	 */
	
	public double get_distance(String tblName,int treeID1,int treeID2,int p,int q) throws Exception
	{
		
		double d;
		Connection[] con = Configuration.getCons(4);
		AdjacencyEncForest f = new AdjacencyEncForest(con[0], con[1],
				new MySqlInsertBuffer(), tblName);
		LblValTree tr = f.loadTree(treeID1);
		LblTree ltr1 = LblValTree.getLblTree(tr);
		tr = f.loadTree(treeID2);
		LblTree ltr2 = LblValTree.getLblTree(tr);
		distance.TreeDist distFunc = null;
		distFunc = new distance.PQGramDist(p, q, false);
		System.out.println(" tree1  "+ltr1.toString());
		System.out.println(" tree2  "+ltr2.toString());
		d = distFunc.treeDist(ltr1, ltr2);
		//System.out.println("d=" + d);
		//return d;
		return d;
		
		
	}
	
	public static void main(String[] args) throws Exception {
		
		if (args.length != 5) {
			System.err.println("  Lookup a tree with <treeID> in a <forest> using a pq-gram index.");
			System.err.println("  params: forest treeID p q tau");
			System.exit(-1);
		}
		
		FileOutputStream out; // declare a file output object
        PrintStream p1; // declare a print stream object
		String tblName = args[0];
		int treeID = Integer.parseInt(args[1]);
		int p = Integer.parseInt(args[2]);
		int q = Integer.parseInt(args[3]);
		int tau = Integer.parseInt(args[4]);
		try
		{
			
			
		// Create a new file output stream
        // connected to "myfile.txt"
        out = new FileOutputStream("C:\\Intern09\\outputs\\lookup.txt",true);
        // Connect print stream to the output stream
        p1 = new PrintStream( out );
		
		
		// connect to database
		// connect to database
		Connection[] con = Configuration.getCons(4);
		
		AdjacencyEncForest f = new AdjacencyEncForest(con[0], con[1],
				new MySqlInsertBuffer(), tblName);

		// get tree size
		String qry =
			"SELECT " + f.getAtbTreeID() + ", COUNT(" + f.getAtbNodeID() + ") AS cnt FROM `" 
			+ f.getTblName() + "` WHERE " + f.getAtbTreeID() + "=" + treeID + " GROUP BY " + f.getAtbTreeID();
		ResultSet rsSize = SQLTools.executeQuery(f.getStatement(), qry,
				"Selecting treeIDs from " + f.getTblName() + ".");
		if (!rsSize.next()) {
			System.err.println("Tree " + treeID + " not found in `" + tblName + "`.");
			System.exit(-2);
		}
		int size = rsSize.getInt("cnt");
		
		// create index and do lookup
		SQLTools.DEBUG = false;
		
		String idxName = tblName + "_Idx_" + p + q;
		Index idx = new Index(idxName, con[2], p, q, new hash.StringHash(4));
		
		String idxNameLookup = tblName + "_lookup_" + p + q;
		Index idxLookup = new Index(idxNameLookup, con[3], 
				p, q, new hash.StringHash(4));
		
		System.out.println("# Looking up tree (" + size + " nodes) with id " + treeID + " in forest "
				+ tblName + " using the " + p + "," + q + "-gram index `"
				+ idxName + "` (created on the fly)...");
		p1.print("# Looking up tree (" + size + " nodes) with id " + treeID + " in forest "
				+ tblName + " using the " + p + "," + q + "-gram index `"
				+ idxName + "` (created on the fly)...");
		p1.println();
		System.out.println("# index creation time [ms], lookup time [ms], total [ms]");		
		p1.print("# index creation time [ms], lookup time [ms], total [ms]");
		p1.println();
		long overall_start = System.currentTimeMillis();
		
		// index creation
		System.err.println("Creating " + p + "," + q + "-gram indices...");
		p1.println("Creating " + p + "," + q + "-gram indices...");
		// lookup tree
		idxLookup.reset();
		idxLookup.addIndex();
		DeltaProfile deltaProf = 
			new DeltaProfile(f, p, q, idxLookup.getHf(), 
					"P_tmp", "Q_tmp", new MySqlInsertBuffer(), true);
		deltaProf.createProfile(treeID);
		idxLookup.add(deltaProf);
		// forest
		idx.buildFromScratch(f);

		long startLookup = System.currentTimeMillis();
		long indexTime =  startLookup - overall_start;
		
		// lookup
		System.err.println("Doing lookup...");
		p1.println("Doing lookup...");
		idx.lookup(idxLookup, tau);
		
		System.out.println(indexTime + " " + (System.currentTimeMillis() - startLookup) + " " +
				(System.currentTimeMillis() - overall_start));
		p1.print(indexTime + " " + (System.currentTimeMillis() - startLookup) + " " +
				(System.currentTimeMillis() - overall_start));
		p1.println();
		}
		
		catch(Exception e)
		{
			System.out.println("Error occured - check the code");
			e.printStackTrace();
		}
		
	}
}
