/*
 * Created on Sep 30, 2005
 */

package vldb2006;
import index.*;
import random.*;
import sqltools.SQLTools;
import adjacencyenc.*;

import java.sql.*;

import mysqltools.MySqlInsertBuffer;
import configuration.*;
/**
 * @author augsten
 */
public class TreeUpdateSeq {
	
	/**
	 * Changes tree and writes Log for numEditOp edit operations. 
	 * Half of them are random inserts, half of them random deletes.
	 * @param f
	 * @param treeID
	 * @param numEditOps
	 * @throws SQLException
	 */
	public static void editSequence(AdjacencyEncForest f, int treeID, int numEditOps) 
	throws SQLException {
		f.setWriteLog(true);
		int maxNodeID = f.getFreeNodeID(treeID, f.getStatement());
		RandomVal random = new RandomVal(1);
		RandomVal random2 = new RandomVal(RandomVal.GAUSS, 1);
		for (int i = 0; i < numEditOps; i++) {
			int nodeID = random.getInt(0, maxNodeID);
			if (nodeID % 2 == 1) { // odd nodeIDs, is never root (nodeID=0)
				f.renameNode(new Integer(treeID), new Integer(nodeID), "R" + i);
				//System.err.println(f.getEditLog().getLast());
			} else {
				int fanout = f.getChildCount(treeID, nodeID);
				int k, m;
				if (fanout == 0) {
					k = 1;
					m = 0;
				} else {
					k = random.getInt(1, fanout);
					// try to avoid to many 0s for m...
					m = random2.getInt(0, 2*(fanout - k));
					if (m < 0) {
						m = 0;
					}
					if (m > fanout - k) {
						m = fanout - k;
					}
				}
				f.insertNode(new Integer(treeID), f.getFreeNodeID(treeID, f.getStatement()), 
						"I" + i, new Integer(nodeID), k, m);
				//System.err.println(f.getEditLog().getLast() + " (fanout: " + fanout + ")");
			}
		}
	}
	
	/**
	 * 
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception {
		if (args.length != 5) {
			System.err.println("\nApplies <numEditOps> to tree <treeID> in <tblName>.\n"
					+ "Assumes a pq-gram index <tblName>_Idx_<p><q> being already there.\n"
					+ "Further a table <tblName>_bak is assumed (copy of <tblName>).\n"
					+ "The edit operations will be executed on the tree, \n" 
					+ "and then the index is updated from the log.\n");
			System.err.println("  params: tblName treeID p q numEditOps\n");
			System.exit(-1);
		}
		String tblName = args[0];
		int treeID = Integer.parseInt(args[1]);
		int p = Integer.parseInt(args[2]);
		int q = Integer.parseInt(args[3]);
		int numEditOps = Integer.parseInt(args[4]);
		
		// connect to database
		// connect to database
		Connection[] con = Configuration.getCons(2);
		Connection out = con[0];
		Connection in = con[1];
		
		AdjacencyEncForest f = new AdjacencyEncForest(out, in,
				new MySqlInsertBuffer(), tblName);
//		AdjacencyEncForest f_bak = new AdjacencyEncForest(out, in,
//				new MySqlInsertBuffer(), tblName + "_bak");
		String idxName = tblName + "_Idx_" + p + q;
	//	Index idx = new Index(idxName, out, p, q, new hash.StringHash(4));
		Index idx = new Index(idxName, out, p, q, new hash.PrefixHash(4));
		
		SQLTools.DEBUG = false;
		long start = System.currentTimeMillis();
		System.err.println("Doing edit operations on tree...");
		editSequence(f, treeID, numEditOps);
		System.err.println("...took me " + (System.currentTimeMillis() - start) + "ms.\n");

		start = System.currentTimeMillis();
		System.err.println("Updating index...");
		idx.update(f);
		System.err.println("...took me " + (System.currentTimeMillis() - start) + "ms.");
				
//		start = System.currentTimeMillis();
//		System.err.println("Restoring original tree...");
//		f.undoEditLog();
//		System.err.println("...took me " + (System.currentTimeMillis() - start) + "ms.");
//		
//		if (f.equals(f_bak)) {
//			System.err.println("The restore operation was successful!");
//		} else {
//			System.err.println("Something went WRONG in the restore operation...");
//		}
			
		
	}

}
