/*
 * Created on Jan 4, 2008
 */
package distance;

import java.util.Enumeration;
import java.util.NoSuchElementException;

import tree.LblTree;

public class BinaryBranchDist extends ProfileDist {

	
	public BinaryBranchDist(boolean normalized) {
		super(normalized);
	}
	
	@Override
	public Profile createProfile(LblTree t) {
		VectorProfile prof = new VectorProfile(100, 100);
		Enumeration<LblTree> breadthFirst = t.breadthFirstEnumeration();
		while (breadthFirst.hasMoreElements()) {
			LblTree currentNode = breadthFirst.nextElement();
			LblTree child;
			try {
				child = (LblTree)currentNode.getFirstChild();
			} catch (NoSuchElementException e) {
				child = new LblTree(PQGram.NULL_SYMBOL, t.getTreeID());
			}
			LblTree sibling = (LblTree)currentNode.getNextSibling();
			if (sibling == null) { sibling = new LblTree(PQGram.NULL_SYMBOL, t.getTreeID()); }
			PQGram binaryBranch = new PQGram(new String[] {currentNode.getLabel(), child.getLabel()}, 
					new String[] {sibling.getLabel()});
			prof.add(binaryBranch);			
		}
		return prof;
	}

	/**
	 * Just an example program.
	 * @param args no arguments - change program inline...
	 */
	public static void main(String[] args) {

		if (args.length != 2) {
			System.out.println("params: tree1 tree2");
			System.out.println("--> trees in string encoding WITH TREE-ID, e.g., 1:{a{b}{c}} 2:{x{b}{c{d}}}");
			System.exit(-1);
		}
		LblTree t1 = LblTree.fromString(args[0]);
		LblTree t2 = LblTree.fromString(args[1]);

		BinaryBranchDist d = new BinaryBranchDist(false);
		d.treeDist(t1, t2);
		t1.prettyPrint();
		System.out.println(d.getProfile(t1));
		t2.prettyPrint(); 
		System.out.println(d.getProfile(t2));
		System.out.println("binary branch distance = " + (d.treeDist(t1, t2) / 4.0));
		d.setNormalized(true);
		System.out.println("normalized binary branch distance = " + d.treeDist(t1, t2));
		
		EditDist edist = new EditDist(false);
		System.out.println("edit distance = " + edist.treeDist(t1, t2));
		edist.setNormalized(true);
		System.out.println("normalized edit distance = " + edist.treeDist(t1, t2));
		
		
	}

	
}
