package distance;

import java.util.*;

import tree.LblTree;

public class CompactGraph {
    NodeData[] nodes;
    int nextNode;
    int[][] arcs;
    LinkedList[] K; // map from G to t1 and t2

    public CompactGraph(int size) {
	nodes = new NodeData[size];
	arcs = new int[size][];
	K = new LinkedList[size];
	nextNode = 0;
    }

    public void add(LblTree n, boolean reverseMapping) {
	NodeData nd = (NodeData)n.getTmpData();
	nodes[nextNode] = nd;
	nd.setCompactGraph(nextNode);
	arcs[nextNode] = new int[n.getChildCount()];
	int i = 0;	
	for (Enumeration e = n.children(); e.hasMoreElements();) {
	    LblTree child = (LblTree)e.nextElement();
	    arcs[nextNode][i]=((NodeData)child.getTmpData()).getCompactGraph();
	    i++;
	}
	if (K[nextNode] == null) {
	    K[nextNode] = new LinkedList();
	}
	if (reverseMapping) {
	    K[nextNode].add(n);
	}
	nextNode++;
    }

    public LinkedList getK(int i) {
	return K[i];
    }

    public void addToK(int i, LblTree n) {
	K[i].add(n);
    }

    public int size() {
	return nextNode;
    }

    public boolean equalsNode(int i, LblTree n) {
	return ((NodeData)n.getTmpData()).equals(nodes[i]);	
    }

    public boolean equalsHeight(int i, LblTree n) {
	return ((NodeData)n.getTmpData()).height==nodes[i].height;	
    }

    public boolean checkChildren(int k, LblTree n) {
	if (n.getChildCount() == arcs[k].length) {
	    for (int i = 0; i < arcs[k].length; i++) {
		if (arcs[k][i] != 
		    ((NodeData)((LblTree)n.getChildAt(i)).getTmpData()).getCompactGraph()) {
		    return false;
		}		
	    }
	    return true;
	} else {
	    return false;
	}
    }

    public void prettyPrint() {
	for (int i = 0; i < size(); i++) {
	    System.out.print((i+1) + ":'" + nodes[i].label + "' (");
	    for (int j = 0; j < arcs[i].length; j++) {		
		System.out.print((arcs[i][j]+1) + ",");
	    }
	    System.out.print(") - K: ");
	    System.out.println(K[i]);
	}
    }
    
}
