package distance;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.NoSuchElementException;

import tree.LblTree;

public class InclusionDist extends EditBasedDist {

	public short[][] inc;
	
	public static final short INCLUDED = 1;
	public static final short NOT_INCLUDED = -1;
	public static final short NOT_COMPUTED = 0;
	
	public InclusionDist() {
		super(false);
	}
	
	private int addPostorderNumbers(LblTree t, int id) {
		for (Enumeration<LblTree> e = t.children(); e.hasMoreElements(); ) {
			id = addPostorderNumbers(e.nextElement(), id);
		}
		t.setTmpData(new Integer(id++));
		return id;		
	}
	
	private int getPost(LblTree t) {
		return ((Integer)t.getTmpData()).intValue();
	}
	
	private short allChildrenIncluded(LblTree t1, LblTree t2) {
		Enumeration<LblTree> seq2 = t2.children();
		for (Enumeration<LblTree> seq1 = t1.children(); seq1.hasMoreElements(); ) {
			LblTree c1 = seq1.nextElement();
			LblTree c2;
			do {
				try {
					c2 = seq2.nextElement();
				} catch (NoSuchElementException e) {
					return NOT_INCLUDED;
				}
			} while ((seq2.hasMoreElements()) && (included(c1, c2) == NOT_INCLUDED));
		}
		return INCLUDED;
	}
	
	
	private short included(LblTree t1, LblTree t2) {
		short result = inc[getPost(t1)][getPost(t2)];
		if (result != NOT_COMPUTED) {
			return result;
		}
		if (t1.getLabel().equals(t2.getLabel())) {
			result = allChildrenIncluded(t1, t2);
		} else {
			result = NOT_INCLUDED;
		}
		inc[getPost(t1)][getPost(t2)] = result;
		return result;
	}
	
	@Override
	public double nonNormalizedTreeDist(LblTree t1, LblTree t2) {
		// add postorder numbers to trees (start with 0)
		addPostorderNumbers(t1, 0);
		addPostorderNumbers(t2, 0);
		// matrix for storing results of subproblems
		inc = new short[getPost(t1) + 1][getPost(t2) + 1];
		for (int i = 0; i < inc.length; i++) {
			Arrays.fill(inc[i], NOT_COMPUTED);
		}
		
		
		short isIncluded = included(t1, t2);
		
		// delete consecutive IDs from tree nodes
		t1.clearTmpData();
		t2.clearTmpData();
		return isIncluded;
	}

	public static void main(String[] args) {

		if (args.length != 2) {
			System.out.println("params: t1 t2");
			System.out.println("  is trees t1 included in t2");
			System.out.println();
			System.out.println("Trees are given as strings with braces, e.g. {a{b}{c}} is\n");
			LblTree.fromString("{a{b}{c}}").prettyPrint();
			System.exit(-1);
		}

		LblTree t1 =  
			LblTree.fromString(args[0]);
		LblTree t2 = 
			LblTree.fromString(args[1]);
		System.out.println("tree t1:");
		t1.prettyPrint();
		System.out.println("tree t2:");
		t2.prettyPrint();

		InclusionDist d = new InclusionDist();
		
		short dist = (short)d.treeDist(t1, t2);
		
		for (int i = 0; i < d.inc.length; i++) {
			for (int j = 0; j < d.inc[i].length; j++) {
				short x = d.inc[i][j];
				if (x == NOT_COMPUTED) {
					System.out.print("* ");
				} else {
					System.out.print(x);
				}
			}
			System.out.println();
		}

		System.out.println(dist == InclusionDist.INCLUDED ? "t1 is included in t2." : "t1 is NOT included in t2.");	
	}

	@Override
	public double profile_dis(Profile p1, Profile p2) {
		// TODO Auto-generated method stub
		return 0;
	}

}


