/*
 * Created on Jan 17, 2005
 */
package distance;

/**
 * @author Varunesh
 */

public class PQGram implements Comparable {
	
	public static final String NULL_SYMBOL = "*";
	
	private String[] pNodes;
	private String[] qNodes;
	
	public PQGram(String[] pNodes, String[] qNodes) {
		this.pNodes = pNodes;
		this.qNodes = qNodes;
		for (int i = 0; i < pNodes.length; i++) {
			if (pNodes[i] == null) {
				pNodes[i] = NULL_SYMBOL;
			}
		}
		for (int i = 0; i < qNodes.length; i++) {
			if (qNodes[i] == null) {
				qNodes[i] = NULL_SYMBOL;
			}
		}
	}
		
	@Override
	public String toString() {
		String res = "(";
		for (int i = 0; i < pNodes.length; i++) {
			res += pNodes[i];
			if (i < pNodes.length - 1) {
				res += ",";
			}
		}
		res += "){";
		for (int i = 0; i < qNodes.length; i++) {
			res += qNodes[i];
			if (i < qNodes.length - 1) {
				res += ",";
			}
		}
		res += "}";
		return res;
	}
	
	private static int compare(String str1, String str2) {
		if ((str1 == null) && (str2 == null)) {
			return 0;
		} 
		if ((str1 != null) && (str2 == null)) {
			return Integer.MAX_VALUE;
		}
		if ((str1 == null) && (str2 != null)) {
			return Integer.MIN_VALUE;
		}
		return str1.compareTo(str2);
		
	}
	
	public String[] getPNodes() {
		return pNodes;
	}
	
	public String[] getQNodes() {
		return qNodes;
	}
	
	public String[] getNodes() {
		String[] nodes = new String[getP() + getQ()];
		System.arraycopy(pNodes, 0, nodes, 0, getP());
		System.arraycopy(qNodes, 0, nodes, getP(), getQ());
		return nodes;
	}
	
	public int getP() {
		return pNodes.length;
	}
	
	public int getQ() {
		return qNodes.length;
	}
	
	/**
	 * compares qq-grams in the following order:
	 * <ol>
	 * <li>p-nodes (=ancestors, farest first)</li>
	 * <li>q-nodes (=children from left to right)</li>
	 * </ol>
	 */
	
	public int compareTo(Object o) {
		PQGram pqg = (PQGram)o;
		String[] qNodes2 = pqg.getQNodes();
		String[] pNodes2 = pqg.getPNodes();
		int cmp = 0;
		int i;
		i = 0;
		while ((cmp == 0) && (i < pNodes.length)) {
			cmp = compare(pNodes[i], pNodes2[i]);
			i++;
		}
		i = 0;
		while ((cmp == 0) && (i < qNodes.length)) {
			cmp = compare(qNodes[i], qNodes2[i]);
			i++;
		}
		return cmp;
	}
	
}