/*
 * Created on 24-Apr-06
 */
package distance;

import hash.FixedLengthHash;

import java.util.LinkedList;
import java.util.ListIterator;

import tree.LblTree;

public class Register {

	LinkedList reg;
	
	public Register(int size) {
		reg = new LinkedList();
		for (int i = 0; i < size; i++) {
			reg.add(null);
		}
	}

	public static Register getPPart(LblTree node, int p) {
		int k = 0;
		LblTree[] regArr = new LblTree[p];
		for (int i = 0; i < p; i++) {
			regArr[i] = node;
			if (node != null) {
				node = (LblTree)node.getParent();
				k++;
			}
		}
		Register reg = new Register(p);
		for (int i = k - 1; i >= 0; i--) {
			reg.shift(regArr[i]);
		}
		return reg;
	}
	
	public void shift(LblTree node) {
		reg.removeFirst();
		reg.addLast(node);
	}
	public void shiftBack(LblTree node) {
		reg.removeLast();
		reg.addFirst(node);
	}
	
	public LblTree[] getRegister() {
		return (LblTree[])reg.toArray(new LblTree[reg.size()]);
	}
	
	public String[] getAsString(FixedLengthHash hf) {
		String[] s = new String[this.getSize()];
		int i = 0;
		for (ListIterator it = reg.listIterator(); it.hasNext();) {
			LblTree t = (LblTree)it.next();
			if (t == null) {
				if (hf == null) {
					s[i] = "*";
				} else {
					s[i] = hf.getNullNode().toString();
				}
			} else {
				if (hf == null) {
					s[i] = t.getLabel();
				} else {
					s[i] = t.getNodeHash(hf).toString();
				}
			}
			i++;
		}
		return s;
	}
	
	public int getSize() {
		return reg.size();
	}
	
	@Override
	public String toString() {
		String s = "[";
		for (ListIterator it = reg.listIterator(); it.hasNext();) {
			LblTree t = (LblTree)it.next();
			if (t != null) {
				s += t.showNode();
			} else {
				s += "null";
			}
			if (it.hasNext()) {
				s += ",";
			}
		}
		s += "]";
		return s;
	}
	
	public Register sorted() {
		LblTree[] regArr = (LblTree[])reg.toArray(new LblTree[this.getSize()]);
		int k = 0;
		for (int i = 0; i < regArr.length; i++) {
			if (regArr[i] == null) {
				regArr[i] = new LblTree("" + Character.MAX_VALUE, 0);
			} else {
				k++;
			}
		}
		java.util.Arrays.sort(regArr);
		Register res = new Register(this.getSize());
		for (int i = k - 1; i >= 0; i--) {
			res.shiftBack(regArr[i]);
		}
		return res;
	}
	
}
