/*
 * Created on Jul 11, 2008
 */
package distance;

import java.io.IOException;
import java.io.Writer;
import java.util.Enumeration;

import tree.LblTree;

public class SvenDistFromProfile extends TreeDist {

	int p,q;
	
	public SvenDistFromProfile(int p, int q, boolean normalized) {
		super(normalized);
		this.p = p;
		this.q = q;
	}

	/**
	 * Return the String profile stored in tree t. If t stores no String profile,
	 * or the profile is not of the correct type, null is returned.
	 * 
	 * This implementation gets the profile with getTmpData() and checks, 
	 * whether it is of type String. 
	 *  
	 * @param t tree
	 * @return string profile of t, if t stores a valid profile, 
	 *         null otherwise
	 */
	public static String getStoredProfile(LblTree t) {
		String p = null;
		if (t.getTmpData() != null) {
			try {
				p = (String)t.getTmpData();
			} catch (ClassCastException e) {
				p = null;
			}
		} 
		return p;
	}
	
	/**
	 * Returns the profile of the tree. If no valid profile is stored in the 
	 * tree (i.e. getStoredProfile returns null), it is caluclated using
	 * createProfile().
	 * 
	 * @param t
	 * @return Profile
	 */
	public static String getProfile(LblTree t, int p, int q) {
		String prof = getStoredProfile(t);
		if (prof == null) {
			prof = createProfile(t, p, q);
		} 
		return prof;
	}

	
	
	/**
	 * Calculates a new profile for a tree.
	 * 
	 * @param t tree
	 * @return profile
	 */
	public static String createProfile(LblTree t, int p, int q) {
		PQGramDist pqgDist = new PQGramDist(p, q, false);
		Profile profPQ = pqgDist.createProfile(t);
		StringBuffer profile = new StringBuffer();
		for (int i = 0; i < profPQ.size(); i++) {
			profile.append(profPQ.elementAt(i).toString() + "\n");
		}
		return profile.toString(); 
		
	}
	
	@Override
	public double treeDist(LblTree t1, LblTree t2) {
		String prof1 = getProfile(t1, this.p, this.q);
		String prof2 = getProfile(t2, this.p, this.q);
		double dist = CrossParsing.crossparse(prof1,prof2);
//		System.out.println(
//				"prof1: " + prof1.substring(0, Math.min(prof1.length(), 100)) + "\n" +  
//				"prof2: " + prof2.substring(0, Math.min(prof2.length(), 100)) + "\n" + 
//				"dist: " + dist + "\n");
		System.out.println(t1.getTreeID() + " <--> " + t2.getTreeID());
		return dist;		
	}
	
	public static String replaceSpaces(String s) {
		if (s.length() == 0) {
			return "_";
		}
		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < s.length(); i++) {
			if (s.charAt(i) == ' ') {
				sb.append('_');
			} else {
				sb.append(s.charAt(i));
			}
		}
		return sb.toString();
	}
	
	/**
	 * Replaces the numbers 0..9 in a string with the 
	 * respective English word. All other characters will
	 * not be changed, e.g. '12.3' --> 'onetwo.three'. 
	 * @param num input string with numeric characters
	 * @return num with numeric characters replaced
	 */
	public static String spellOutNumber(String num) {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < num.length(); i++) {
			char ch = num.charAt(i);
			switch (ch) {
			case '0': sb.append("zero"); break;
			case '1': sb.append("one"); break;
			case '2': sb.append("two"); break;
			case '3': sb.append("three"); break;
			case '4': sb.append("four"); break;
			case '5': sb.append("five"); break;
			case '6': sb.append("six"); break;
			case '7': sb.append("seven"); break;
			case '8': sb.append("eight"); break;
			case '9': sb.append("nine"); break;
			default:
				sb.append(ch);
			}
		}
		return sb.toString();		
	}

	
	public static void lblTreeToXML(LblTree t, Writer out, boolean spelloutNumbers)  
	throws IOException {
		String label = t.getLabel();
		label = replaceSpaces(label);
		if (spelloutNumbers) {
			label = spellOutNumber(label);
		} else {
			try {
				Double.parseDouble(label.charAt(0) + "");
				label = "_" + label;
			} catch (NumberFormatException e){
			}
		} 		
		if (t.isLeaf()) {
			out.write("<" + label + "/>");
			return;
		}
		out.write("<" + label + ">");
		for (Enumeration<LblTree> e = t.children(); e.hasMoreElements();) {
			lblTreeToXML(e.nextElement(), out, spelloutNumbers);			
		}
		out.write("</" + label + ">");
	}

	@Override
	public double profile_dis(Profile p1, Profile p2) {
		// TODO Auto-generated method stub
		return 0;
	}


}
