/*
 * Created on Mar 4, 2008
 */
package distance;

import tree.LblTree;
import java.util.*;

class Membership {
	private int[] membership;
	private int pointer = 0;
	private double descWeight;
	
	public Membership(int frequency) {
		this.membership = new int[frequency];
		descWeight = 1;
	}
	
	public double getDescWeight() {
		return descWeight;
	}
	
	public void decrementDescWeight(double d) {
		descWeight -= d;
	}
	
	public int getFrequency() {
		return membership.length;
	}
	
	public void addPQGram(int pos) {
		membership[pointer] = pos;
		pointer++;
	}
	
	public double getNodeWeight(VectorProfile p) {
		double sum = 0;
		for (int i = 0; i < this.pointer; i++) {
			sum += ((WeightedPQGram)p.elementAt(membership[i])).weight;
		}
		return sum;
	}
	
}

public class WeightedPQGramDist extends ProfileDist {
		
	private int p, q;	
	
	public WeightedPQGramDist(int p, int q, boolean normalized) {
		super(normalized);
		this.p = p;
		this.q = q;
	}

	/**
	 * Weighted intersection between two profiles.
	 * 
	 * @param p1 
	 * @param p2
	 * @return
	 */
	@Override
	public double intersect(Profile p1, Profile p2) {
		int i1 = 0, i2 = 0;
		double insec = 0;
		while ((i1 < p1.size()) && (i2 < p2.size())) {
			WeightedPQGram w1 = (WeightedPQGram)p1.elementAt(i1);
			WeightedPQGram w2 = (WeightedPQGram)p2.elementAt(i2);
			int cmp = w1.unweightedCompareTo(w2);
			if (cmp == 0) {
				i1++;
				i2++;
				insec += Math.min(w1.getWeight(), w2.getWeight());
			} else if (cmp < 0) {
				i1++;
			} else {
				i2++;
			}
		}
		return insec;
	}
	
	@Override
	public Profile createProfile(LblTree t) {
		int leafCnt = 0;
		int nonLeafCnt = 0;
		// give successive IDs to nodes, starting with 0
		for (Enumeration e = t.preorderEnumeration(); e.hasMoreElements();) {
			LblTree node = ((LblTree)e.nextElement());
			node.setNodeID(leafCnt + nonLeafCnt);
			if (node.isLeaf()) { 
				leafCnt++;
			} else {
				nonLeafCnt++;
			}
		}	
		Vector prof = new Vector(2 * leafCnt + this.q * nonLeafCnt - 1, 1);		
		this.getPQGrams(prof, t);
		// define datastructure for weights
		Membership[] membership = new Membership[leafCnt + nonLeafCnt];
		for (Enumeration e = t.preorderEnumeration(); e.hasMoreElements();) {
			LblTree node = ((LblTree)e.nextElement());
			int f = node.getChildCount();
			membership[node.getNodeID()] = new Membership(node.isRoot() ? f + q - 1 : f + 2 * q - 1);
		}						
		// copy pq-grams
		WeightedVectorProfile pqProf = new WeightedVectorProfile (prof.size(), 0);
		for (int i = 0; i < prof.size(); i++) {
			String[] stem = new String[this.p];
			String[] base = new String[this.q];
			for (int k = 0; k < this.p; k++) {
				LblTree nd = ((LblTree)((Vector)prof.elementAt(i)).elementAt(k));
				if (nd != null) {
					stem[k] = nd.getLabel();
				} else {
					stem[k] = PQGram.NULL_SYMBOL;
				}
			}
			for (int k = p; k < p + q; k++) {
				LblTree nd = ((LblTree)((Vector)prof.elementAt(i)).elementAt(k));
				if (nd != null) {
					base[k - p] = nd.getLabel();
				} else {
					base[k - p] = PQGram.NULL_SYMBOL;
				}
			}
			pqProf.add(new WeightedPQGram(stem, base, 1));
		}
		// compute weights
 	 	// frequency weighting
//		for (int i = 0; i < prof.size(); i++) {
//			int phi = 0;
//			int nonNulls = 0;			
//			for (int k = 0; k < this.p + this.q; k++) {
//				LblTree nd = ((LblTree)((Vector)prof.elementAt(i)).elementAt(k));
//				if (nd != null) {
//					int id = nd.getNodeID();
//					phi = phi + membership[id].getFrequency();
//					nonNulls++;
//					membership[id].addPQGram(i);
//				}
//			}			
//			double weight = (double)1/phi;
//			((WeightedPQGram)pqProf.elementAt(i)).setWeight(weight);
//		}
//        // max weighting
		for (int i = 0; i < prof.size(); i++) {
			int max = 0;
			int nonNulls = 0;			
			for (int k = 0; k < this.p + this.q; k++) {
				LblTree nd = ((LblTree)((Vector)prof.elementAt(i)).elementAt(k));
				if (nd != null) {
					int id = nd.getNodeID();
					int nodePhi = membership[id].getFrequency();
					max = (max < nodePhi) ? nodePhi : max; 
					nonNulls++;
					membership[id].addPQGram(i);
				}
			}			
			double weight = (double)1/max;
			((WeightedPQGram)pqProf.elementAt(i)).setWeight(weight);
		}
 	 	// parent weighting
//		for (int i = 0; i < prof.size(); i++) {
//			WeightedPQGram pqgram = ((WeightedPQGram)pqProf.elementAt(i));
//			LblTree root = ((LblTree)((Vector)prof.elementAt(i)).elementAt(0));
//			int phi = membership[root.getNodeID()].getFrequency();
//			double descWeight = membership[root.getNodeID()].getDescWeight();
//			double weight;
//			if (root.isRoot()) {
//				weight = descWeight/phi;
//			} else {
//				weight = descWeight/(phi - q);				
//			}
//			pqgram.setWeight(weight);
//			membership[root.getNodeID()].addPQGram(i);
//			for (int k = 1; k < this.p + this.q; k++) {
//				LblTree nd = ((LblTree)((Vector)prof.elementAt(i)).elementAt(k));
//				if (nd != null) {
//					int id = nd.getNodeID();
//					membership[id].decrementDescWeight(weight);
//					membership[id].addPQGram(i);
//				}
//			}						
//		}
		// compute maximum node weight
		double max = 0;
		for (int nd = 0; nd < membership.length; nd++) {
			double weight = membership[nd].getNodeWeight(pqProf);
			max = (weight > max) ? weight : max;			
		}
		// adapt pq-gram weights
		double sumPQGramWeights = 0;
		for (int i = 0; i < pqProf.size(); i++) {
			WeightedPQGram g = (WeightedPQGram)pqProf.elementAt(i);
			g.setWeight(g.getWeight() / (2 * max)); // scale to [0 .. 0.5]
			g.setWeight(1); // scale to [0 .. 0.5]
			sumPQGramWeights += g.getWeight(); 
		}
		// statistics about the weighting
		max = 0;
		double sum = 0;
		double min = Double.MAX_VALUE;
		for (int nd = 0; nd < membership.length; nd++) {
			double weight = membership[nd].getNodeWeight(pqProf);
			max = (weight > max) ? weight : max;
			min = (weight < min) ? weight : min;
			sum += weight;
		}
		System.err.println("min/max node weight: " + min + "/" + max);
		System.err.println("sum node weights: " + sum);
		System.err.println("sum pq-gram weights: " + sumPQGramWeights 
				+ " (max possible: " + (membership.length / 2.0) + ")");
		return pqProf;
	}
	
	private static Vector concat(Vector a, LinkedList b) {
		Vector c = new Vector(a.size() + b.size());
		for (int i = 0; i < a.size(); i++) {
			c.add(a.elementAt(i));
		}
		for (ListIterator it = b.listIterator(); it.hasNext();) {
			c.add(it.next());
		}
		return c;
	}

	/**
	 * Recursive method to calculate pq-gram profile of a tree. Is called 
	 * by {@link #createProfile(LblTree)}.
	 * 
	 * @param prof pq-grams will be added to this profile
	 * @param node start with this node
	 * @param p
	 * @param q
	 */
	private void getPQGrams(Vector prof, LblTree node) {
		if ((q <= 0) || (p <= 0)) {
			throw new RuntimeException("Both p and q must be greater than zero!");
		}
		
		
		// initialize stem
		Vector<LblTree> stem = new Vector<LblTree>(p);
		stem.setSize(p);
		LblTree n = node;
		int i = p - 1;
		while ((n != null) && (i >= 0)) {
			stem.setElementAt(n, i);
			n = (LblTree)n.getParent();
			i--;
		}
		
		LinkedList<LblTree> base = new LinkedList<LblTree>();
		for (i = 0; i < q; i++) {
			base.add(null);
		}	

		for (Enumeration e = node.children(); e.hasMoreElements();) {
			base.add((LblTree)e.nextElement());
			base.removeFirst();			
			prof.add(concat(stem, base));	
		}	
		if (node.getChildCount() > 0) {
			for (i = 0; i < q - 1; i++) {
				base.add(null);
				base.removeFirst();
				prof.add(concat(stem, base));
			}
		} else {
			prof.add(concat(stem, base));
		}
		for (Enumeration e = node.children(); e.hasMoreElements();) {
			getPQGrams(prof, (LblTree)e.nextElement());
		}   		

	}
	
}
