/*
 * Created on 20-Apr-06
 */
package intervalenc;

import java.sql.Connection;
import java.sql.SQLException;

import mysqltools.MySqlInsertBuffer;
import mysqltools.MySqlTools;

import sqltools.SQLTools;

public class ModifyForest {
	
	public static boolean printProgress = false;

	private ModifyForest() {
	}

	/**
	 * Creates a new forest with the subtrees rooted in the children of a root node.
	 * Tree IDs in the new forest will be 1, 2, 3, ...
	 * 
	 * @param sf where the tree is you want to split
	 * @param treeID the tree to split
	 * @param tf where to store the new forest
	 * @return number of copied trees
	 */
	public static int createForestFromTree(IntervalEncForest sf,
			int treeID, IntervalEncForest tf) throws SQLException {
		tf.reset();
		IntervalEncNode root = sf.getRootNode(treeID);
		if (root == null) {
			return 0;
		}
		int currentTreeID = 1;
		IntervalEncNode nd = sf.getNode(treeID, root.getLft() + 1); 
		while (nd != null) {
			String qry = 
				"INSERT INTO " + tf.getTblName() + 
				" SELECT " + 
				currentTreeID + " AS " + tf.getAtbTreeID() + "," +
				sf.getAtbLabel() + " AS " + tf.getAtbLabel() + "," + 
				sf.getAtbValue() + " AS " + tf.getAtbValue() + "," + 
				sf.getAtbLft() + "-" + (nd.getLft()) + " AS " + tf.getAtbLft() + "," + 
				sf.getAtbRgt() + "-" + (nd.getLft()) + " AS " + tf.getAtbRgt() +  
			    " FROM " + sf.getTblName() +
				" WHERE " + sf.getAtbLft() + ">=" + nd.getLft() +
				" AND " + sf.getAtbLft() + "<=" + nd.getRgt() + 
				// using lft (instead of rgt) here speeds up the query a lot! 
				" AND " + sf.getAtbTreeID() + "=" + treeID;
			int size = SQLTools.executeUpdate(tf.getStatement(), qry, 
					"Splitting tree id=" + treeID + " into forest '" + tf.getTblName() + "'.");
			if ((SQLTools.DEBUG) || (printProgress)) {
				System.out.println("Inserted tree " + currentTreeID + " with " + size + " nodes...");
			}
			currentTreeID++;
			nd = sf.getNode(treeID, nd.getRgt() + 1);
		}
		return currentTreeID - 1;
	}

	/**
	 * Copy tree with treeID from forest sf to forest tf.
	 * @param sf
	 * @param treeID
	 * @param tf
	 * @return size of tree copied.
	 * @throws SQLException
	 */
	public static int copyTree(IntervalEncForest sf, int treeID, 
			IntervalEncForest tf) throws SQLException {
		String qry = 
			"INSERT INTO " + tf.getTblName() + 
			" SELECT * " + 
		    " FROM " + sf.getTblName() +
			" WHERE " + sf.getAtbTreeID() + "=" + treeID;
		return SQLTools.executeUpdate(tf.getStatement(), qry,
				"Copying tree id=" + treeID + " from forest '" + sf.getTblName() 
				+ "' to forest '" + tf.getTblName() + "'.");
	}

	/**
	 * Copy forest sf to forest tf. Index on (treeID, lft) is created.
	 * @param sf
	 * @param treeID
	 * @param tf
	 * @return size of forest copied.
	 * @throws SQLException
	 */
	public static long copyForest(IntervalEncForest sf,  
			IntervalEncForest tf) throws SQLException {
		tf.reset();
		String qry = 
			"INSERT INTO " + tf.getTblName() + 
			" SELECT * " + 
		    " FROM " + sf.getTblName();
		long forestSize =  SQLTools.executeUpdate(tf.getStatement(), qry,
				"Copying forest '" + sf.getTblName() 
				+ "' to forest '" + tf.getTblName() + "'.");
		tf.createPreorderIndex();
		return forestSize;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		SQLTools.DEBUG = true;
		MySqlTools.loadDriver();
		System.err.println("connecting to database...(augsten@localhost.test)");
		Connection out = MySqlTools.getConnection("localhost", "test", 
				"user", "password");
		Connection in = MySqlTools.getConnection("localhost", "test", 
				"user", "password");

		// connect to forests
		IntervalEncForest sf = new IntervalEncForest(
				out, in, 
				new MySqlInsertBuffer(), "forest");
		IntervalEncForest tf = new IntervalEncForest(
				out, in, 
				new MySqlInsertBuffer(), "forest_new");
		
		tf.reset();
		int n = ModifyForest.copyTree(sf, 1000, tf);
		System.out.println("Created " + n + " new trees");
	}
}
