/*
 * Created on 20-Apr-06
 */
package intervalenc;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.TreeSet;
import java.util.Vector;

import mysqltools.MySqlInsertBuffer;
import mysqltools.MySqlTools;

import random.RandomVal;
import sqltools.SQLTools;

public class NoiseGenerator {

	private NoiseGenerator() {
	}
	
	public static void renameLeafNodes(IntervalEncForest f, 
			int treeID, int labelCount, int n, RandomVal r) throws SQLException {
		String qry = 
			"SELECT * FROM " + f.getTblName() + " WHERE " +
			f.getAtbTreeID() + "=" + treeID + " AND " +
			f.getAtbRgt() + "=" + f.getAtbLft() + "+1";
		ResultSet rs = SQLTools.executeQuery(f.getStatement(), qry, 
					"Getting all leaves of tree with id=" + treeID + ".");
		Vector leaves = new Vector(n, n);
		while (rs.next()) {
			leaves.add(new IntervalEncNode(
					rs.getInt(f.getAtbTreeID()),
					rs.getString(f.getAtbLabel()),
					rs.getString(f.getAtbValue()),
					rs.getInt(f.getAtbLft()),
					rs.getInt(f.getAtbRgt())));
		}
		if (leaves.size() < n) {
			n = leaves.size();
		}
		TreeSet renamed = new TreeSet();
		int i = 0;
		while (i < n) {
			int pos = r.getInt(0, leaves.size() - 1);
			IntervalEncNode nd = (IntervalEncNode)leaves.elementAt(pos);
			if (!renamed.contains(new Integer(nd.getLft()))) {
				String lbl = r.getString(labelCount);
				f.renameNode(nd.getTreeID(), nd.getLft(), lbl);
				renamed.add(new Integer(nd.getLft()));
				i++;
			} 
		}
	}

	public static void deleteLeafNodes(IntervalEncForest f, 
			int treeID, int labelCount, int n, RandomVal r) throws SQLException {
		String qry = 
			"SELECT * FROM " + f.getTblName() + " WHERE " +
			f.getAtbTreeID() + "=" + treeID + " AND " +
			f.getAtbRgt() + "=" + f.getAtbLft() + "+1";
		ResultSet rs = SQLTools.executeQuery(f.getStatement(), qry, 
					"Getting all leaves of tree with id=" + treeID + ".");
		Vector leaves = new Vector(n, n);
		while (rs.next()) {
			leaves.add(new IntervalEncNode(
					rs.getInt(f.getAtbTreeID()),
					rs.getString(f.getAtbLabel()),
					rs.getString(f.getAtbValue()),
					rs.getInt(f.getAtbLft()),
					rs.getInt(f.getAtbRgt())));
		}
		if (leaves.size() < n) {
			n = leaves.size();
		}
		int i = 0;
		while (i < n) {
			int pos = r.getInt(0, leaves.size() - 1);
			IntervalEncNode nd = (IntervalEncNode)leaves.elementAt(pos);
				f.deleteNode(nd.getTreeID(), nd.getLft());
				leaves.remove(pos);
				for (int k = pos; k < leaves.size(); k++) {
					IntervalEncNode nk = ((IntervalEncNode)leaves.elementAt(k));
					nk.setLft(nk.getLft() - 2);
				}
				i++;
		}
	}	
	
	
	public static void renameAnyNodes(IntervalEncForest f, 
			int treeID, int labelCount, int n, RandomVal r) throws SQLException {
		String qry = 
			"SELECT * FROM " + f.getTblName() + " WHERE " +
			f.getAtbTreeID() + "=" + treeID;
		ResultSet rs = SQLTools.executeQuery(f.getStatement(), qry, 
					"Getting all leaves of tree with id=" + treeID + ".");
		Vector leaves = new Vector(n, n);
		while (rs.next()) {
			leaves.add(new IntervalEncNode(
					rs.getInt(f.getAtbTreeID()),
					rs.getString(f.getAtbLabel()),
					rs.getString(f.getAtbValue()),
					rs.getInt(f.getAtbLft()),
					rs.getInt(f.getAtbRgt())));
		}
		if (leaves.size() < n) {
			n = leaves.size();
		}
		TreeSet renamed = new TreeSet();
		int i = 0;
		while (i < n) {
			int pos = r.getInt(0, leaves.size() - 1);
			IntervalEncNode nd = (IntervalEncNode)leaves.elementAt(pos);
			if (!renamed.contains(new Integer(nd.getLft()))) {
				String lbl = r.getString(labelCount);
				f.renameNode(nd.getTreeID(), nd.getLft(), lbl);
				renamed.add(new Integer(nd.getLft()));
				i++;
			} 
		}
//		System.out.println("renamed " + renamed.size() + " nodes");
	}

	
	public static void main(String[] args) throws Exception {
		SQLTools.DEBUG = false;
		MySqlTools.loadDriver();
		System.err.println("connecting to database...(augsten@localhost.test)");
		Connection out = MySqlTools.getConnection("localhost", "test", 
			 	"user", "password");
		Connection in = MySqlTools.getConnection("localhost", "test", 
			 	"user", "password");
		
		// connect to forests
		IntervalEncForest f = new IntervalEncForest(
				out, in, 
				new MySqlInsertBuffer(), "tree1");
		NoiseGenerator.renameLeafNodes(f, 1, 10, 200, new RandomVal());
	}

}
