package matching;

import marriage.Association;
import marriage.Man;
import marriage.PersonList;
import marriage.Relation;
import marriage.Woman;
import distmat.DistMatrix;

public class  StblMarriage extends MatchingAlgo {
	
	public StblMarriage() {
		super();
	}
	
	public static boolean DEBUG = false;
	
    public static PersonList getWomenList(int[] prefList, Woman[] women) {
		int pos = prefList[prefList.length - 1];    		
    	PersonList person = new PersonList(women[pos], null);
    	for (int i = prefList.length - 2; i >= 0; i--) {
    		pos = prefList[i];    		
    		person = new PersonList(women[pos], person);
    	}
    	return person;
    }

    public static PersonList getMenList(int[] prefList, Man[] men) {
		int pos = prefList[prefList.length - 1];    		
    	PersonList person = new PersonList(men[pos], null);
    	for (int i = prefList.length - 2; i >= 0; i--) {
    		pos = prefList[i];    		
    		person = new PersonList(men[pos], person);
    	}
    	return person;
    }
	
	@Override
	public Match[] match(DistMatrix dm) {
	   	Man[] men = new Man[dm.getRowNum()];
    	for (int row = 0; row < dm.getRowNum(); row++) {
    		men[row] = new Man(row);
    	}

    	Woman[] women = new Woman[dm.getColNum()];
    	for (int col = 0; col < dm.getColNum(); col++) {
    		women[col] = new Woman(col, dm.getIdCol(col));
    	}
    	
    	for (int row = 0; row < dm.getRowNum(); row++) {
    		int[] pl = dm.getRowPrefs(row);
    		men[row].list = getWomenList(pl, women); 
    	}

    	for (int col = 0; col < dm.getColNum(); col++) {
    		int[] pl = dm.getColPrefs(col);
    		women[col].list = getMenList(pl, men); 
    	}
    	
    	PersonList eligible = new PersonList(men[0], null);
    	for (int row = 1; row < dm.getRowNum(); row++) {
    		eligible = new PersonList(men[row], eligible);
    	}

    	Relation r = findMarriages(eligible);
    	
    	Association cur = r.list;
    	Match[] matches = new Match[Math.min(dm.getRowNum(), dm.getColNum())];    	    	
    	int row = 0;
    	while ((cur != null) && (cur.next != null)) {
    		cur = cur.next;
    		int pos1 = ((Man)cur.range).pos;
    		int pos2 = ((Woman)cur.domain).pos;
    		matches[row] = new Match(dm.getIdRow(pos1), dm.getIdCol(pos2));
    		row++;
    	}	
		return matches;
	}
	
    public static Relation findMarriages(PersonList eligible) {
        Relation couples = new Relation();
        while (eligible != null) {
            Man m = (Man) eligible.person;
            Woman w = m.topPick();
            if (w == null) {
    			eligible = eligible.next;
    			if (DEBUG) {
					System.out.println("No woman left for " + m);
				}
            } else {
            	if (DEBUG) {
					System.out.println("" + m + " proposes to " + w);
				}
            	if (w.likes(m)) {
                	if (DEBUG) {
						System.out.print("  she accepts ");
					}
            		Man oldHusband = (Man) couples.lookup(w);
            		if (oldHusband == null) {
						eligible = eligible.next;
					} else {
                    	if (DEBUG) {
							System.out.print("(dumping " + oldHusband + ")");
						}
            			eligible = new PersonList(oldHusband, eligible.next);
            		}
            		couples.map(w,m);
            		w.trimList(m);
                	if (DEBUG) {
						System.out.println("\n  but she still prefers: " + w.getList());
					}
            	}
            	m.scratchTop();
            }
        }
        return couples;
    }
    
 }
