package random;
import java.util.Random;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Arrays;
import tree.LblTree;

public class TreeNoise {
	
	private TreeNoise() {
	}
	
	/**
	 * @param tree tree to change (make deepCopy, if you do not want the original tree to be changed!
	 * @param n number of nodes to change
	 * @param r a random value object (for example 'new RandomVal()')
	 */
	public static void changeAnyNodes(LblTree tree, int labelCount, int n, RandomVal r) {
		int nodes = tree.getNodeCount();
		if (nodes < n) {
			n = nodes;
		}
		int[] nodesToChange = new int[nodes];
		Arrays.fill(nodesToChange, 0);
		int found = 0;
		while (found < n) {
			int node = r.getInt(0, nodes - 1);
			if (nodesToChange[node] == 0) {
				found++;
				nodesToChange[node] = -1;
			}
		}
		int currentNode = 0;
		for (Enumeration e = tree.breadthFirstEnumeration(); e.hasMoreElements();) {    
			LblTree node = (LblTree)e.nextElement();
			if (nodesToChange[currentNode] == -1) {
				change(node, new Random(), labelCount);
			}
			currentNode++;
		}
	}
	
	/**
	 * @param tree tree to change (make deepCopy, if you do not want the original tree to be changed!
	 * @param n number of nodes to delete
	 * @param r a random value object (for example 'new RandomVal()')
	 */
	public static void deleteAnyNodes(LblTree tree, int n, RandomVal r) {
		int nodes = tree.getNodeCount();
		if (nodes <= n) {  // can not delete more nodes than the tree has, and can not delete root node!
			n = nodes - 1;
		}
		for (int i = 0; i < n; i++) {
			deleteAny(tree, r);
		}
	}
	
	
	public static void changeAny(LblTree tree, int labelCount, RandomVal r) {
		int nodes = tree.getNodeCount();
		int nodeToChange = r.getInt(0, nodes - 1); 
		System.out.println("Node to change " + nodeToChange);
		int currentNode = 0;
		for (Enumeration e = tree.breadthFirstEnumeration(); e.hasMoreElements();) {    
			LblTree n = (LblTree)e.nextElement();
			if (currentNode == nodeToChange) {
				change(n, new Random(), labelCount);
			}
			currentNode++;
		}
	}
	
	public static void deleteAny(LblTree tree, RandomVal r) {
		int nodes = tree.getNodeCount();
		int nodeToChange = r.getInt(1, nodes - 1);  // can not delete root node!
		int currentNode = 0;
		for (Enumeration e = tree.breadthFirstEnumeration(); e.hasMoreElements();) {    
			LblTree n = (LblTree)e.nextElement();
			if (currentNode == nodeToChange) {
				delete(n);
			}
			currentNode++;
		}
	}
	
	
	public static void changeUniform(LblTree tree, double percentage) {
		if (percentage == 0) {
			return;
		}
		int nodes = tree.getNodeCount();
		Random r = new Random();
		for (Enumeration e = tree.breadthFirstEnumeration(); e.hasMoreElements();) {
			LblTree n = (LblTree)e.nextElement();
			if (Math.random() < percentage) {
				change(n, r, nodes);
			}
		}
	}
	
	
	public static void deleteUniform(LblTree tree, double percentage) {
		if (percentage == 0) {
			return;
		}
		int capacity = (int)(tree.getNodeCount() * percentage * 1.2) + 1;
		Vector delNodes = new Vector(capacity, (int)(capacity * 0.2) + 1);
		for (Enumeration e = tree.depthFirstEnumeration(); e.hasMoreElements();) {
			LblTree n = (LblTree)e.nextElement();
			if (Math.random() < percentage) {
				delNodes.add(n);
			}
		}
		for (int i = 0; i < delNodes.size(); i++) {
			delete((LblTree)delNodes.elementAt(i));
		}	
	}
	
	public static void moveUniform(LblTree tree, double percentage) {
		if (percentage == 0) {
			return;
		}
	}
	
	public static void change(LblTree tree, Random r, int labelCount) {
		tree.setLabel(Integer.toString(r.nextInt(labelCount), Character.MAX_RADIX));
	}
	
	/** 
	 * Deletes a node and inserts the children of the deleted node between the left
	 * and the right sibling of that node.
	 *
	 * If the deleted node is a root node, one of the children of this node randomly is
	 * choosen to be the new root node. The children of the new root are inserted
	 * between its former left and richt siblings.
	 * If root is leaf, nothing happens.
	 * 
	 * (Internally the root node is renamed and one of the children is deleted!)
	 */
	
	public static void delete(LblTree tree) {
		if (tree.isRoot()) {
			if (tree.isLeaf()) {
				return;
			}
			int r = (new Random()).nextInt(tree.getChildCount());
			LblTree newRoot = (LblTree)tree.getChildAt(r);
			tree.setLabel(newRoot.getLabel());
			tree = newRoot;
		}
		LblTree parent = (LblTree)tree.getParent();
		int id = parent.getIndex(tree);
		int cc = tree.getChildCount();
		for (int i = 0; i < cc; i++) {
			parent.insert((LblTree)tree.getChildAt(0), id + 1 + i);
		}
		parent.remove(id);
	}
	
	public static void move(LblTree tree) {
	}
	
	
}
