/**
 * 
 */
package tasmalgos;

import intervalenc.Cursor;
import intervalenc.IntervalEncCursor;
import intervalenc.IntervalEncForest;
import intervalenc.IntervalEncNode;

import java.sql.SQLException;

import tasmTED.NodeDistPair;
import tree.LblTree;
import tree.LblValTree;
import tree.LblValTreeCursor;
import distance.IndividualWeighting;
import distance.SubtreeWeighting;
import distance.WeightedEditDist;
import configuration.*;
/**
 * @author naugsten
 *
 */
public class IETasm {
	
	public static LblValTree subtreeInPreorder(IntervalEncForest f, int treeID, int lft) throws SQLException {
		IntervalEncNode n = f.getNode(treeID, lft);
		System.out.print(" (" + n.getLabel() + "," + n.getLft() + "," + n.getRgt() + ") ");
		String qry = 
			"SELECT " + f.getAtbTreeID() + "," + f.getAtbLabel() + ", " + f.getAtbValue() + "," 
			+ f.getAtbLft() + "," + f.getAtbRgt()
			+ " FROM " + f.getTblName() 
			+ " WHERE " + f.getAtbTreeID() + "=" + treeID  
			+ " AND " + f.getAtbLft() + ">=" + n.getLft() 
			+ " ORDER BY " + f.getAtbTreeID() + "," + f.getAtbLft();
		Cursor cursor = new IntervalEncCursor(f.getStreamStatement().executeQuery(qry), f);
		cursor.next();
		return tasmalgos.TASM.getSubtree(cursor);
	}	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		TASM.DEBUG = true;
		String strParams = "params: tblIEForest treeID query k x y [weight0 weigth1 weight2 ...]\n";
		if (args.length < 6) {
			System.err.print(strParams);
			System.exit(-1);
		}		
		String tblName = args[0];
		int treeID = Integer.parseInt(args[1]);
		int k = Integer.parseInt(args[3]);
		float x = Float.parseFloat(args[4]);
		float y = Float.parseFloat(args[5]);
		float[] weights = new float[args.length - 6];
		for (int i = 6; i < args.length; i++) {
			weights[i - 6] = Float.parseFloat(args[i]);
		}
		LblValTree query = new LblValTree(LblTree.fromString(args[2]));
		
		Configuration config = new Configuration("." + IETasm.class.getSimpleName());
		Configuration.loadDrivers();
		IntervalEncForest f = 
			new IntervalEncForest(config.getConnection(), config.getConnection(),
					Configuration.getInsertBuffer(), tblName);
		
		WeightedEditDist dist;
		if (weights.length == 0) {
			dist = new WeightedEditDist(
					new SubtreeWeighting(x), new SubtreeWeighting(y), false);
		} else {
			dist = new WeightedEditDist(
					new IndividualWeighting(weights), new SubtreeWeighting(y), false);
		}
		System.out.println("query: " + query);
		query.prettyPrint();
		System.out.println(dist);
		WallClock wc = new WallClock(System.out);
		wc.start("");
		NodeDistPair[] result = TASM.tasm(new LblValTreeCursor(query), 
				f.treeInPreorder(treeID), 
				k, dist);
//		Result[] result = new Result[] {
//				new Result(13478437, -1), 
//				new Result(13478471, -1), 
//				new Result(13478577, -1), 
//				new Result(13478607, -1), 
//		};
		System.out.print("Time for subtree computation: ");
		wc.printStop();
		for (int i = 0; i < result.length; i++) {
			System.out.print("node ID: " + result[i].getNodeID() + " dist: " + result[i].getDist());
			LblValTree resTree = IETasm.subtreeInPreorder(f, treeID, result[i].getNodeID());
			System.out.println(" " + resTree);
		}

	}

}
