/*
 * maybe you want to reuse the code?
 */

package tree;

import java.util.Enumeration;
import java.util.LinkedList;
import java.util.Random;

import random.RandomInt;
import tree.LblTree;

public class AlterTree {
	
	private AlterTree() {
	}
	
	/**
	 * @param randomPos random variable to draw change
	 * positions. Return values must be somewhere between 0 and
	 * (nodes.length - 1) inclusive.
	 * @param numberOfChanges exactly this number of changes will be
	 * done. No duplicate changes. This means, that if you choose this 
	 * value higher than the number of values possibly returned by randomPos,
	 * the method will loop for ever.
	 */
	public static void changeLabels(Enumeration enums, RandomInt randomPos,
			int numberOfChanges) {	
		changeLabels(getNodesArray(enums), randomPos, numberOfChanges);
	}
	
	public static void changeLabels(LblTree[] nodes, RandomInt randomPos,
			int numberOfChanges) {	
		
		// change a node if it is not already changed
		int changesDone = 0;
		while (changesDone < numberOfChanges) {
			int pos = randomPos.getInt();
			if ((pos >= 0) && (pos < nodes.length)) {
				LblTree n = nodes[pos];
				if (n != null) {
					changeLabel(n);
					nodes[pos] = null;	
					changesDone++;
				}
			}
		}	
	}
	
	public static void deleteLabels(Enumeration enums, RandomInt randomPos,
			int numberOfDeletions) {
		deleteLabels(getNodesArray(enums), randomPos, numberOfDeletions);
	}
	
	public static void deleteLabels(LblTree[] nodes, RandomInt randomPos,
			int numberOfDeletions) {
		// delete a label if it is not already deleted
		int deletionsDone = 0;
		while (deletionsDone < numberOfDeletions) {
			int pos = randomPos.getInt();
			if ((pos >= 0) && (pos < nodes.length)) {
				LblTree n = nodes[pos];
				if ((n != null) && (!n.isRoot())) {
					deleteNode(n);
					nodes[pos] = null;	
					deletionsDone++;
				}
			}
		}
	}
	
	// ******* some tools *********
	
	private static LblTree[] getNodesArray (Enumeration enums) {
		LinkedList nodesList = new LinkedList();
		while (enums.hasMoreElements()) {
			nodesList.add(enums.nextElement());	    
		}
		LblTree[] nodes = new LblTree[nodesList.size()];
		return (LblTree[])nodesList.toArray(nodes);
	}
	
	private static void changeLabel(LblTree n) {
		// n.setLabel("\\textbf{" + n.getLabel() + "}");
		n.setLabel(n.getLabel() + "_changed");		
	}
	
	public static void deleteNode(LblTree n) {
		if (n.isRoot()) {
			if (n.isLeaf()) {
				return;
			}
			int posOfNewRoot = (new Random()).nextInt(n.getChildCount());
			LblTree newRoot = (LblTree)n.getChildAt(posOfNewRoot);
			n.setLabel(newRoot.getLabel());
			deleteNode(newRoot);
		} else {
			LblTree parent = (LblTree)n.getParent();
			int parentsChildID = parent.getIndex(n);
			int childCount = n.getChildCount();
			for (int i = 0; i < childCount; i++) {
				parent.insert((LblTree)n.getChildAt(0), 
						parentsChildID + 1 + i);
			}
			parent.remove(parentsChildID);
		}
	}
	
	public static void renameToChildCount(LblTree n) {
		n.setLabel("" + n.getChildCount());
		for (Enumeration e = n.children(); e.hasMoreElements();) {
			renameToChildCount((LblTree)e.nextElement());
		}
	}
	
	public static void renameNonLeafToChildCount(LblTree n) {
		if (!n.isLeaf()) {
			n.setLabel("--" + n.getChildCount() + "--");
			for (Enumeration e = n.children(); e.hasMoreElements();) {
				renameNonLeafToChildCount((LblTree)e.nextElement());
			}
		}
	}

	public static void siblingCount(LblTree n) {
		if (n.isRoot()) {
			n.setLabel("r");
		} else {			
			LblTree p = (LblTree)n.getParent();
			int pos = p.getIndex(n);
			n.setLabel("*" + pos + "*");
		}
		for (Enumeration e = n.children(); e.hasMoreElements();) {
			siblingCount((LblTree)e.nextElement());
		}
	}
	
	public static void levelAndSibling(LblTree n, int level) {
		if (n.isRoot()) {
			n.setLabel("0 - 0");
			level = 0;
		} else {			
//			LblTree p = (LblTree)n.getParent();
//			int pos = p.getIndex(n);
//			n.setLabel(pos + " - " + level);
		}
		for (Enumeration e = n.children(); e.hasMoreElements();) {
			levelAndSibling((LblTree)e.nextElement(), level + 1);
		}
	}

	
}
