package util;

import java.util.Map;
import java.util.Hashtable;

/**
 * This provides a way of using small int values to represent String labels, 
 * as opposed to storing the labels directly.
 */
public class LabelDictionary {
	public static final int NO_KEY = Integer.MIN_VALUE;
	private int count;
	private Map<String,Integer> StrInt;
	private Map<Integer,String> IntStr;

	/**
	 * Creates a new blank dictionary.
	 *  
	 * @throws Exception
	 */
	public LabelDictionary() {
		count = 0;
		StrInt = new Hashtable<String,Integer>();
		IntStr = new Hashtable<Integer,String>();
	}
	
	
	/**
	 * Adds a new label to the dictionary if it has not been added yet. 
	 * Returns the ID of the new label in the dictionary.
	 * 
	 * @param label add this label to the dictionary if it does not exist yet
	 * @return ID of label in the dictionary
	 */	
	public int store(String label){
		if (StrInt.containsKey(label)) {
			return (StrInt.get(label).intValue());
		}

		Integer intKey = new Integer(count++);
		StrInt.put(label, intKey);
		IntStr.put(intKey, label);
		
		return intKey.intValue();		
	}
	
	/**
	 * 	 Returns the key ID of the a given label if the label exists in the dictionary, 
	 *   return {@link #NO_KEY} otherwise.
	 * 
	 * @param label return the dictionary if it does not exist yet
	 * @return ID of label if it exists in the dictionary, {@link #NO_KEY} otherwise
	 */	
	public int getLabelID(String label){
		if (StrInt.containsKey(label)) {
			return (StrInt.get(label).intValue());
		} else {
			return NO_KEY;
		}
	}
	
	
	/**
	 * Returns the label with a given ID in the dictionary.
	 *	
	 * @param labelID 
	 * @return the label with the specified labelID, or null if this dictionary contains no label for labelID
	 */
	public String read(int labelID){
		return IntStr.get(new Integer(labelID));		
	}
	
}