/*
 * Created on Mar 17, 2005
 */
package xml;

import tree.LblTree;
import tree.Node;
import org.xml.sax.SAXException;

/**
 * @author augsten
 */
public class LblTreeHandler extends MyDefaultHandler {

	private LblTree tree;
	
	public LblTreeHandler() {
		super();
	}

	public static LblTree parseFromFile(String filename) {
		return parseFromFile(filename, true, true);
	}
	
	public static LblTree parseFromFile(String filename, 
			boolean validating, 
			boolean nameSpaceAware) {
        LblTreeHandler handler = new LblTreeHandler();
        Parser.parseFile(filename, handler, validating, nameSpaceAware);        
		return handler.tree;
	}
	
	/* (non-Javadoc)
	 * @see org.xml.sax.ContentHandler#startDocument()
	 */
	@Override
	public void startDocument() throws SAXException {
		tree = new LblTree("dummy_root", Node.NO_TREE_ID);
	}
		
	/* (non-Javadoc)
	 * @see org.xml.sax.ContentHandler#endDocument()
	 */
	@Override
	public void endDocument() throws SAXException {
		if (tree.isLeaf()) {
			tree = null;
		} else {
			// remove dummy root
			LblTree dummy_root = (LblTree)tree.getRoot();
			tree = (LblTree)dummy_root.getFirstChild(); 
		    tree.removeFromParent();
		}
	}

	/* (non-Javadoc)
	 * @see xml.MyDefaultHandler#startElement(java.lang.String)
	 */
	@Override
	public void startElement(String elementName) {
		tree.add(new LblTree(elementName, tree.getTreeID()));
		tree = (LblTree)tree.getLastChild();
	}
    	
	/* (non-Javadoc)
	 * @see xml.MyDefaultHandler#endElement()
	 */
	@Override
	public void endElement(String elementName) {
		tree = (LblTree)tree.getParent();
	}

	/* (non-Javadoc)
	 * @see xml.MyDefaultHandler#foundAttrNode(java.lang.String, java.lang.String)
	 */
	@Override
	public void foundAttrNode(String name, String value) {
		LblTree attrNode = new LblTree(name, tree.getTreeID());
		attrNode.add(new LblTree(value, tree.getTreeID()));
		tree.add(attrNode);
     }
	
	/* (non-Javadoc)
	 * @see xml.MyDefaultHandler#foundTextNode(java.lang.String)
	 */
	@Override
	public void foundTextNode(String text) {
		tree.add(new LblTree(text, tree.getTreeID()));
	}
	    
    public static void main(String argv[])
    {
        if (argv.length != 1) {
            System.err.println("Usage: cmd filename");
            System.exit(1);
        }
        LblTreeHandler.parseFromFile(argv[0], false, false).prettyPrint();
        
    }
}
