package xml;

import java.io.FileReader;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.XMLReaderFactory;


public class SvenFamilyLine extends WriterHandler
{

	String siblingsOnLevel[][];
	int currentpos[];
	int currentLevel;
	boolean lastTagWasEndTag;

	public static String parseFromFile(String filename, 
			boolean validating, 
			boolean nameSpaceAware) {
		StringWriter writer = new StringWriter();
		SvenFamilyLine handler = new SvenFamilyLine(writer);
		Parser.parseFile(filename, handler, validating, nameSpaceAware);
		return handler.outWriter.toString();
	}


	public static void main (String args[])
	throws Exception
	{
		XMLReader xr = XMLReaderFactory.createXMLReader();
		SvenFamily handler = new SvenFamily(new StringWriter());
		xr.setContentHandler(handler);
		xr.setErrorHandler(handler);

		// Parse each file provided on the
		// command line.
		for (int i = 0; i < args.length; i++) {
			FileReader r = new FileReader(args[i]);
			xr.parse(new InputSource(r));
		}
	}


	public SvenFamilyLine (Writer out)
	{
		super();

		int i;

		currentLevel = 0;
		currentpos = new int[200];
		for(i = 0; i < 200; i++) {
			currentpos[i] = 0;
		}
		siblingsOnLevel = new String[200][200];

		this.outWriter = out;
	}

	@Override
	public void reset() {
		currentLevel = 0;
		currentpos = new int[200];
		for(int i = 0; i < 200; i++) {
			currentpos[i] = 0;
		}
		siblingsOnLevel = new String[200][200];
		
	}


	////////////////////////////////////////////////////////////////////
	// Event handlers.
	////////////////////////////////////////////////////////////////////



	@Override
	public void startDocument ()
	{
//		System.out.println("Start document");
		lastTagWasEndTag = false;
	}


	@Override
	public void endDocument ()
	{	
//		System.out.println("End document");
		int i;
		try {
			if(lastTagWasEndTag) {
				for(i = 0; i < currentpos[currentLevel]; i++) {
					outWriter.write(
							siblingsOnLevel[i][currentLevel]);
					outWriter.write("\n");
				}
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}


	@Override
	public void startElement (String uri, String name,
			String qName, Attributes atts)
	{
//		System.out.println("Start element");
		int i;


		siblingsOnLevel[currentpos[currentLevel]][currentLevel] = 
			new String(qName);
		for(i = 0; i < atts.getLength(); i++) {
			siblingsOnLevel[currentpos[currentLevel]][currentLevel] += 
				new String(atts.getLocalName(i));
		}

	siblingsOnLevel[currentpos[currentLevel]][currentLevel] += '\n';

		currentpos[currentLevel]++;
		currentLevel++;

		lastTagWasEndTag = false;
	}


	@Override
	public void endElement (String uri, String name, String qName)
	{
//		System.out.println("End element");
		int i;

		try {
			if(lastTagWasEndTag) {
				for(i = 0; i < currentpos[currentLevel]; i++) {
					outWriter.write(
							siblingsOnLevel[i][currentLevel]);
				}
				currentpos[currentLevel] = 0;
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
		currentLevel--;
		lastTagWasEndTag = true;
	}


	@Override
	public void characters (char ch[], int start, int length)
	{
		/*
	out.write("Characters:    \"");
	for (int i = start; i < start + length; i++) {
	    switch (ch[i]) {
	    case '\\':
		out.write("\\\\");
		break;
	    case '"':
		out.write("\\\"");
		break;
	    case '\n':
		out.write("\\n");
		break;
	    case '\r':
		out.write("\\r");
		break;
	    case '\t':
		out.write("\\t");
		break;
	    default:
		out.write(ch[i]);
		break;
	    }
	}
	out.write("\"\n");
		 */
	}


	public Writer getOut() {
		return outWriter;
	}

}

