package xml;

import java.io.FileReader;
import java.io.StringWriter;
import java.io.Writer;

import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.XMLReaderFactory;


public class SvenPQGrams extends WriterHandler
{
	String siblingsOnLevel[][];
	String attributesOnLevel[][];
	String tagNames[];
	String attributeNames[];

	int currentpos[];
	int currentNoOfAttributes[];
	int currentLevel;

	boolean lastTagWasEndTag;

	int p;
	int q;
	
	public static String parseFromFile(String filename, 
			boolean validating, 
			boolean nameSpaceAware) {
		StringWriter writer = new StringWriter();
		SvenPQGrams handler = new SvenPQGrams(writer, 2, 3);
		Parser.parseFile(filename, handler, validating, nameSpaceAware);
		return handler.outWriter.toString();
	}
	
	public static void main (String args[])
	throws Exception
	{
		if(args.length < 3) {
			System.out.println("Usage:  MySAXAppPQ <filename> <p> <q>");
			System.exit(0);
		}

		XMLReader xr = XMLReaderFactory.createXMLReader();
		SvenPQGrams handler = new SvenPQGrams(new StringWriter(), Integer.parseInt(args[1]), 
				Integer.parseInt(args[2]));

		xr.setContentHandler(handler);
		xr.setErrorHandler(handler);

		// parse file
		FileReader r = new FileReader(args[0]);
		xr.parse(new InputSource(r));

	}


	public SvenPQGrams(Writer out, int sizeOfP, int sizeOfQ)
	{
		super();

		this.outWriter = out;
		
		int i;

		p = sizeOfP;
		q = sizeOfQ;
		currentLevel = 0;
		currentpos = new int[10000];
		currentNoOfAttributes = new int[10000];
		tagNames = new String[10000];
		attributeNames = new String[10000];
		for(i = 0; i < 10000; i++) {
			currentpos[i] = 0;
			currentNoOfAttributes[i] = 0;
		}
		siblingsOnLevel = new String[10000][20];
		attributesOnLevel = new String[10000][20];
	}

	@Override
	public void reset() {
	
		int i;

		currentLevel = 0;
		currentpos = new int[10000];
		currentNoOfAttributes = new int[10000];
		tagNames = new String[10000];
		attributeNames = new String[10000];
		for(i = 0; i < 200; i++) {
			currentpos[i] = 0;
			currentNoOfAttributes[i] = 0;
		}
		siblingsOnLevel = new String[10000][20];
		attributesOnLevel = new String[10000][20];		
	}

	////////////////////////////////////////////////////////////////////
	// Event handlers.
	////////////////////////////////////////////////////////////////////


	@Override
	public void startDocument ()
	{
//		System.out.println("Start document");
		lastTagWasEndTag = false;
	}


	@Override
	public void endDocument ()
	{	
//		System.out.println("End document");
	}


	@Override
	public void startElement (String uri, String name,
			String qName, Attributes atts)
	{
//		System.out.println("Start element");
		int i;

		currentNoOfAttributes[currentLevel] = atts.getLength();
		attributeNames[currentLevel] = new String("");
		attributesOnLevel[currentpos[currentLevel]][currentLevel] = new String("");
		for(i = 0; i < currentNoOfAttributes[currentLevel]; i++) {
			attributeNames[currentLevel] += new String(atts.getLocalName(i));
			attributesOnLevel[currentpos[currentLevel]][currentLevel] += 
				new String(atts.getLocalName(i));
		}

		tagNames[currentLevel] = new String(qName);
		siblingsOnLevel[currentpos[currentLevel]][currentLevel] = new String(qName);

		currentpos[currentLevel]++;

		// start processing on level beneath first one
		if(currentLevel != 0) {

			try {
			// output p-part

			for(i = currentLevel - p; i < currentLevel; i++) {
				if(i < 0) {
					outWriter.write("*");
				}
				else {
					outWriter.write(tagNames[i]);
					outWriter.write(attributeNames[i]);
				}
			}

			// output q-part

			for(i = currentpos[currentLevel] - q; i < currentpos[currentLevel]; i++) {
				if(i < 0) {
					outWriter.write("*");
				}
				else {
					outWriter.write(siblingsOnLevel[i][currentLevel]);
					outWriter.write(attributesOnLevel[i][currentLevel]);
				}
			}
			outWriter.write("\n");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		currentLevel++;

		lastTagWasEndTag = false;
	}


	@Override
	public void endElement (String uri, String name, String qName)
	{
//		System.out.println("End element");
		int i;
		int j;

		try {
		if(lastTagWasEndTag) {
			// we've reached the last sibling in a group
			// output the remaining pq-grams

			for(j = 1; j < q; j++) {

				// output p-part

				for(i = currentLevel - p; i < currentLevel; i++) {
					if(i < 0) {
						outWriter.write("*");
					}
					else {
						outWriter.write(tagNames[i]);
						outWriter.write(attributeNames[i]);
					}
				}

				// output q-part

				for(i = currentpos[currentLevel] - q + j; i < currentpos[currentLevel] + j; i++) {
					if((i < 0) || (i >= currentpos[currentLevel])) {
						outWriter.write("*");
					}
					else {
						outWriter.write(siblingsOnLevel[i][currentLevel]);
						outWriter.write(attributesOnLevel[i][currentLevel]);
					}
				}
				outWriter.write("\n");
			}
			currentpos[currentLevel] = 0;
		}
		else {
			// last tag was start tag (which means we've reached a leaf

			// output p-part

			for(i = currentLevel - p; i < currentLevel; i++) {
				if(i < 0) {
					outWriter.write("*");
				}
				else {
					outWriter.write(tagNames[i]);
					outWriter.write(attributeNames[i]);
				}
			}

			// output q-part (consists of q *s in this case)

			for(i = 0; i < q; i++) {
				outWriter.write("*");
			}
			outWriter.write("\n");
		}

		} catch (Exception e) {
			e.printStackTrace();
		}
		currentLevel--;
		lastTagWasEndTag = true;
	}


	@Override
	public void characters (char ch[], int start, int length)
	{
		/*
	System.out.print("Characters:    \"");
	for (int i = start; i < start + length; i++) {
	    switch (ch[i]) {
	    case '\\':
		System.out.print("\\\\");
		break;
	    case '"':
		System.out.print("\\\"");
		break;
	    case '\n':
		System.out.print("\\n");
		break;
	    case '\r':
		System.out.print("\\r");
		break;
	    case '\t':
		System.out.print("\\t");
		break;
	    default:
		System.out.print(ch[i]);
		break;
	    }
	}
	System.out.print("\"\n");
		 */
	}

	/**
	 * Get the writer with the results.
	 * 
	 * @return
	 */
	public Writer getOut() {
		return outWriter;
	}
	
	/**
	 * Substitute the current output with a new one (i.e., reset the output).
	 * 
	 * @param out
	 */
	public void setOut(Writer out) {
		this.outWriter = out;
	}

}

