package xml;
import java.io.FileReader;
import java.io.StringWriter;
import java.io.Writer;

import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.XMLReaderFactory;


public class SvenPath extends WriterHandler
{
	String tagnames[];
	int currentpos;

	public static String parseFromFile(String filename, 
			boolean validating, 
			boolean nameSpaceAware) {
		StringWriter writer = new StringWriter();
		SvenPath handler = new SvenPath(writer);
		Parser.parseFile(filename, handler, validating, nameSpaceAware);
		return handler.outWriter.toString();
	}

	public static void main (String args[])
	throws Exception
	{
		XMLReader xr = XMLReaderFactory.createXMLReader();
		SvenPath handler = new SvenPath(new StringWriter());
		xr.setContentHandler(handler);
		xr.setErrorHandler(handler);

		// Parse each file provided on the
		// command line.
		for (int i = 0; i < args.length; i++) {
			FileReader r = new FileReader(args[i]);
			xr.parse(new InputSource(r));
		}
	}


	public SvenPath (Writer out)
	{
		super();
		currentpos = 0;
		tagnames = new String[2400];
		this.outWriter = out;
	}

	@Override
	public void reset() {				
		currentpos = 0;
		tagnames = new String[2400];
	}	


	////////////////////////////////////////////////////////////////////
	// Event handlers.
	////////////////////////////////////////////////////////////////////


	@Override
	public void startDocument ()
	{
//		System.out.println("Start document");
	}


	@Override
	public void endDocument ()
	{
//		System.out.println("End document");
	}


	@Override
	public void startElement (String uri, String name,
			String qName, Attributes atts)
	{
		int i;

		try {
		for(i = 1; i <= currentpos; i++) {
			outWriter.write(tagnames[i]);
		}
		outWriter.write(qName);
		currentpos++;
		tagnames[currentpos] = new String(qName);

		for(i = 0; i < atts.getLength(); i++) {
			outWriter.write(" " + atts.getLocalName(i));
		}
		outWriter.write("\n");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public void endElement (String uri, String name, String qName)
	{
		currentpos--;
	}


	@Override
	public void characters (char ch[], int start, int length)
	{
		/*
	out.write("Characters:    \"");
	for (int i = start; i < start + length; i++) {
	    switch (ch[i]) {
	    case '\\':
		out.write("\\\\");
		break;
	    case '"':
		out.write("\\\"");
		break;
	    case '\n':
		out.write("\\n");
		break;
	    case '\r':
		out.write("\\r");
		break;
	    case '\t':
		out.write("\\t");
		break;
	    default:
		out.write(ch[i]);
		break;
	    }
	}
	out.write("\"\n");
		 */
	}

	public Writer getOut() {
		return outWriter;
	}

}

