package xml;

import java.io.FileReader;
import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

import org.xml.sax.XMLReader;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.helpers.XMLReaderFactory;


public class SvenTag extends WriterHandler
{

	public static String parseFromFile(String filename, 
			boolean validating, 
			boolean nameSpaceAware) {
		StringWriter writer = new StringWriter();
        SvenTag handler = new SvenTag(writer);
        Parser.parseFile(filename, handler, validating, nameSpaceAware);
		return handler.outWriter.toString();
	}

	public static void main (String args[])
	throws Exception
	{
		XMLReader xr = XMLReaderFactory.createXMLReader();
		SvenTag handler = new SvenTag(new StringWriter());
		xr.setContentHandler(handler);
		xr.setErrorHandler(handler);

		// Parse each file provided on the
		// command line.
		for (int i = 0; i < args.length; i++) {
			FileReader r = new FileReader(args[i]);
			xr.parse(new InputSource(r));
		}
	}


	public SvenTag (Writer out)
	{
		super();
		this.outWriter = out;
	}

	@Override
	public void reset() {				
	}

	////////////////////////////////////////////////////////////////////
	// Event handlers.
	////////////////////////////////////////////////////////////////////


	@Override
	public void startDocument ()
	{
//		System.out.println("Start document");
	}


	@Override
	public void endDocument ()
	{
//		System.out.println("End document");
	}


	@Override
	public void startElement (String uri, String name,
			String qName, Attributes atts)
	{
		int i;
		try {
			if ("".equals (uri)) {
				outWriter.write("<" + qName);
				for(i = 0; i < atts.getLength(); i++) {
					outWriter.write(" " + atts.getLocalName(i));
				}
				outWriter.write(">\n");
			}
			else {
				outWriter.write("<" + uri + qName);
				for(i = 0; i < atts.getLength(); i++) {
					outWriter.write(" " + atts.getLocalName(i));
				}
				outWriter.write(">\n");
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void endElement (String uri, String name, String qName)
	{
		try {
		if ("".equals (uri)) {
			outWriter.write("</" + qName + ">\n");
		} else {
			outWriter.write("</" + uri + name + ">\n");
		}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}


	@Override
	public void characters (char ch[], int start, int length)
	{
		/*
	out.write("Characters:    \"");
	for (int i = start; i < start + length; i++) {
	    switch (ch[i]) {
	    case '\\':
		out.write("\\\\");
		break;
	    case '"':
		out.write("\\\"");
		break;
	    case '\n':
		out.write("\\n");
		break;
	    case '\r':
		out.write("\\r");
		break;
	    case '\t':
		out.write("\\t");
		break;
	    default:
		out.write(ch[i]);
		break;
	    }
	}
	out.write("\"\n");
		 */
	}

	public Writer getOut() {
		return outWriter;
	}

}

